package net.xdclass.shopmanager.controller;

import com.google.ortools.Loader;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.xdclass.shopmanager.model.*;
import net.xdclass.shopmanager.service.SubService;
import net.xdclass.shopmanager.util.JsonData;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static net.xdclass.shopmanager.model.PmSmallPltPosDO.SMALL_PLATE_DIV_NO;
import static net.xdclass.shopmanager.tool.Tool.nvl;
import static net.xdclass.shopmanager.util.Knapsack.solve;

@RestController
@Slf4j(topic = "xt-special1")
@RequestMapping("/Sub")
@Api(tags = "余坯组板模块", value = "获取数据组板相关接口")
public class SubController {
    @Autowired
    private SubService subService;

    private final static Logger logger = LoggerFactory.getLogger(SubController.class);

    //region ***全局变量过程数据***
    private static List<GPreOrder> gltOrder = new ArrayList<>();

    //初次订单分组
    private static List<GGrpOrderFir> gltGrpOrderFir = new CopyOnWriteArrayList<>();

    //二次计算分组
    private static List<GCalGrp> gltGCalGrp = new CopyOnWriteArrayList<>();

    //计算结果数据
    private static List<GGrpCalRslt> gltGGrpCalRslt = Collections.synchronizedList(new ArrayList<>());

    //选择结果数据
    private static List<GGrpCalRslt> gltSelGGrpCalRslt = Collections.synchronizedList(new ArrayList<>());

    //组板板坯表
    private static List<PmPdSlabDO> pmPdSlabDOList = new ArrayList<>();
    //大板粗切位置表
    private static List<PmPdPlateCutDO> pmPdPlateCutDOList = new ArrayList<>();
    //组合子板位置表
    private static List<PmPdBlockPosDO> pmPdBlockPosDOList = new ArrayList<>();
    //组板合同总量表
    private static List<PmPdContSumDO> pmPdContSumDOList = new ArrayList<>();
    //组板小板位置表
    private static List<PmSmallPltPosDO> pmSmallPltPosDOList = new ArrayList<>();
    //命令板坯表
    private static List<PmCmdSlabDO> pmCmdSlabDOList = new ArrayList<>();
    //命令大板表
    private static List<PmCmdLargePltDO> pmCmdLargePltDOList = new ArrayList<>();
    //命令参数表
    private static List<PmCmdContDO> pmCmdContDOList = new ArrayList<>();
    //命令小板表
    private static List<PmCmdSmallPltDO> pmCmdSmallPltDOList = new ArrayList<>();

    // 全局变量，使用 ConcurrentHashMap 和 AtomicInteger 确保线程安全
    private final ConcurrentHashMap<String, AtomicInteger> counterMap = new ConcurrentHashMap<>();

    private static int intOrdNum = 0;

    private static int intGrpNum = 0;

    //endregion

    //region   ***全局变量获取基础数据***

    //板坯信息
    private static List<SlabInfoDO> plistSlab;
    //合同信息
    private static List<PmProContDO> plistOrder;
    //生产合同产品属性表
    private static List<PmProContProdAttrDO> plistPmProContProdAttrDO;
    //合同产线工艺参数表
    private static List<PmContMscTechDO> plistPmContMscTechDO;
    //组板设计规则集
    private static List<PmPdRuleSetDO> plistPmPdRuleSetDO;
    //组板设计规则集
    private static List<PmPdRuleItemDO> plistPmPdRuleItemDO;
    //最小轧制厚度公差范围基准表
    private static List<PmMinThickTolBtDO> plistPmMinThickTolBtDO;
    //钢板轧制规格对应收得率基准表
    private static List<PmPlateSpecYieldBtDO> plistPmPlateSpecYieldBtDO;
    //强度级别对照基准表
    private static List<PmStCodeBtDO> plistPmStCodeBtDO;
    //粗切位置决定基准表
    private static List<PmCutPlaceBtDO> plistPmCutPlaceBtDO;
    //组板常数基准表
    private static List<PmPdConstBtDO> plistPmPdConstBtDO;
    //钢板切缝切头尾范围基准表
    private static List<PmPlateCutLenBtDO> plistPmPlateCutLenBtDO;
    //组板临时收得率基准表
    private static List<PmPdTmpYieldBtDO> plistPmPdTmpYieldBtDO;
    //钢板切边量基准表
    private static List<PmPlateTrimBtDO> plistPmPlateTrimBtDO;
    //余材组板内部钢种以优充次对照表
    private static List<PmSlabReplBtDO> plistPmSlabReplBtDO;
    //热处理炉基准规格表
    private static List<PmHeatTreatFurnaceBtDO> plistPmHeatTreatFurnaceBtDO;
    //特殊钢种余量增加表
    private static List<PmSpeStAddvBtDO> plistPmSpeStAddvBtDO;
    //合同处理成分结果数据
    private static List<QdToLineupResultCheDO> plistQdToLineupResultCheDO;
    //合同处理坯料性能要求
    private static List<QdToLineupResultSlabphyDO> plistQdToLineupResultSlabphyDO;
    //炉次工序成分代表实绩表
    private static List<QeTqHeatcheStandResultDO> plistQeTqHeatcheStandResultDO;
    //炉次待委托表
    private static List<QeTqPonoWaitTestDO> plistQeTqPonoWaitTestDO;
    //坯料低倍委托实绩表
    private static List<QeTqSlabMacroResultDO> plistQeTqSlabMacroResultDO;
    //连铸板坯生产规格基准表
    private static List<PmSlabSpecBtDO> plistPmSlabSpecBtDO;
    //板坯信息表
    private static List<PmPdSlabInfoDO> plistPmPdSlabInfoDO;  //TODO 需要添加此表信息的查询
    //TODO  需要添加 QD_TO_PO_ORDER 的查询记录
    private static Date now;
    //组板履历表
    private  static List<PmSubHisDO>  plistpmSubHisDO=new ArrayList<>();
    //组板订单详细表
    private  static List<PmSubOrdDO> plistpmSubOrdDO=new ArrayList<>();

    //    //加热炉板坯最小长度
    //    private static int gintSlabMinLen;
    //    //加热炉板坯最大长度
    //    private static int gintSlabMaxLen;

    //endregion

    @ApiOperation("求解处理")
    @PostMapping("Solver")
    public JsonData Solver() {
        // 测试数据
        long[] values = {360, 83, 59, 130, 431, 67, 230, 52, 93, 125, 670, 892, 600, 38, 48, 147,
                78, 256, 63, 17, 120, 164, 432, 35, 92, 110, 22, 42, 50, 323, 514, 28, 87, 73, 78, 15, 26,
                78, 210, 36, 85, 189, 274, 43, 33, 10, 19, 389, 276, 312};
        long[][] weights = {{7, 1, 30, 22, 80, 94, 11, 81, 70, 64, 59, 18, 1, 36, 3, 8, 15, 42, 9,
                1, 42, 47, 52, 32, 26, 48, 55, 6, 29, 84, 2, 4, 18, 56, 7, 29, 93, 44, 71, 3, 86, 66, 31,
                65, 1, 79, 20, 65, 52, 13}};
        long[] capacities = {7};
        //初始化方案
        Loader.loadNativeLibraries();
        // 调用 solve 方法来解决问题
        ArrayList<Long> ltInt = solve(values, weights, capacities);
        for (int i = 0; i < ltInt.size(); i++) {
            System.out.println(ltInt.get(i));
        }
        return JsonData.buildSuccess();
    }

    @ApiOperation("数据处理")
    @PostMapping("SubData")
    public JsonData SubData(String strSlab) {

        //获取所有数据
        GetDataInfo(null);

        //数据预处理
        SetPreData();

        logger.info("***初次分组数量***  " + gltGrpOrderFir.size());

        //循环板坯
        plistSlab.forEach(p -> {
                    logger.info("*************板坯开始{}*************  ", p.getSlabNo());
                    try {
                        gltGCalGrp = new ArrayList<>();
                        //二次分组
                        SetGrpDataBySlab(p);

                        logger.info("***二次分组数量***  " + gltGCalGrp.size());

                        if (gltGCalGrp.size() > 0) {
                            gltGGrpCalRslt = new ArrayList<>();

                            gltSelGGrpCalRslt= new ArrayList<>();

                            SetGroupCalRslt(p);

                            logger.info("结果数据板坯号：{}；数量：{}", p.getSlabNo(), gltGGrpCalRslt.size());
                        }
                        //  if (gltGGrpCalRslt.size() > 0) {
                        //      now = new Date();
                        //      SetSaveDate(p);
                        //  }
                    } catch (Exception e) {
                        logger.error("板坯报错：" + p.getSlabNo(), e);
                    }
                    logger.info("*************板坯结束{}*************  ", p.getSlabNo());
                    logger.info("");
                }
        );
        logger.info("*****处理结束*****");
        logger.info("");

        return JsonData.buildSuccess();
    }

    /**
     * 获取数据-多线程
     * 板坯数据
     * 订单数据
     * 配置信息
     */
    public void GetDataInfo(Optional<SubParams> args) {

        logger.info("*****开始获取数据******");

        //region  ***多线程获取基础数据***

        // 创建线程池
        ExecutorService executorService = Executors.newFixedThreadPool(24);

        //初始化方案
        Loader.loadNativeLibraries();

        // 创建多个任务
        List<Callable<Void>> tasks = new ArrayList<>();

        //1板坯数据
        tasks.add(() -> {
            plistSlab = subService.getSlabInfo();
//            logger.info("******1板坯数据获取完毕*******");
            return null;
        });

        //2订单数据
        tasks.add(() -> {
            plistOrder = subService.getOrderInfo();
//            logger.info("******2订单数据获取完毕*******");
            return null;
        });

        //3生产合同产品属性表
        tasks.add(() -> {
            plistPmProContProdAttrDO = subService.getPmProCountProdAttr();
//            logger.info("******3生产合同产品属性表数据获取完毕*******");
            return null;
        });

        //4合同产线工艺参数表
        tasks.add(() -> {
            plistPmContMscTechDO = subService.getPmContMscTech();
//            logger.info("******4合同产线工艺参数表数据获取完毕*******");
            return null;
        });

        //5生产合同产品属性表
        tasks.add(() -> {
            // plistPmProContProdAttrDO = subService.getPmProCountProdAttr();
            Thread.sleep(10000); // 休眠10秒钟
//            logger.info("******5生产合同产品属性表数据获取完毕*******");
            return null;
        });

        //6组板设计规则集
        tasks.add(() -> {
            plistPmPdRuleSetDO = subService.getPmPdRuleSet();
//            logger.info("******6组板设计规则集数据获取完毕*******");
            return null;
        });

        //7组板设计规则集
        tasks.add(() -> {
            plistPmPdRuleItemDO = subService.getPmPdRuleItem();
//            logger.info("******7组板设计规则集数据获取完毕*******");
            return null;
        });

        //8最小轧制厚度公差范围基准表
        tasks.add(() -> {
            plistPmMinThickTolBtDO = subService.getPmMinThickTolBt();
//            logger.info("******8最小轧制厚度公差范围基准表数据获取完毕*******");
            return null;
        });

        //9钢板轧制规格对应收得率基准表
        tasks.add(() -> {
            plistPmPlateSpecYieldBtDO = subService.getPmPlateSpecYieldBt();
//            logger.info("******9钢板轧制规格对应收得率基准表数据获取完毕*******");
            return null;
        });

        //10强度级别对照基准表
        tasks.add(() -> {
            plistPmStCodeBtDO = subService.getPmStCodeBt();
//            logger.info("******10强度级别对照基准表数据获取完毕*******");
            return null;
        });

        //11粗切位置决定基准表
        tasks.add(() -> {
            plistPmCutPlaceBtDO = subService.getPmCutPlaceBt();
//            logger.info("******11粗切位置决定基准表数据获取完毕*******");
            return null;
        });

        //12组板常数基准表
        tasks.add(() -> {
            plistPmPdConstBtDO = subService.getPmPdConstBt();
//            logger.info("******12组板常数基准表数据获取完毕*******");
            return null;
        });

        //13钢板切缝切头尾范围基准表
        tasks.add(() -> {
            plistPmPlateCutLenBtDO = subService.getPmPlateCutLenBt();
//            logger.info("******13钢板切缝切头尾范围基准表数据获取完毕*******");
            return null;
        });

        //14组板临时收得率基准表
        tasks.add(() -> {
            plistPmPdTmpYieldBtDO = subService.getPmPdTmpYieldBt();
//            logger.info("******14组板临时收得率基准表数据获取完毕*******");
            return null;
        });

        //15钢板切边量基准表
        tasks.add(() -> {
            plistPmPlateTrimBtDO = subService.getPmPlateTrimBt();
//            logger.info("******15钢板切边量基准表数据获取完毕*******");
            return null;
        });

        //16余材组板内部钢种以优充次对照表
        tasks.add(() -> {
            plistPmSlabReplBtDO = subService.getPmSlabReplBt();
//            logger.info("******16余材组板内部钢种以优充次对照表数据获取完毕*******");
            return null;
        });

        //17热处理炉基准规格表
        tasks.add(() -> {
            plistPmHeatTreatFurnaceBtDO = subService.getPmHeatTreatFurnaceBt();
//            logger.info("******17热处理炉基准规格表数据获取完毕*******");
            return null;
        });

        //18特殊钢种余量增加表
        tasks.add(() -> {
            plistPmSpeStAddvBtDO = subService.getPmSpeStAddvBt();
//            logger.info("******18特殊钢种余量增加表数据获取完毕*******");
            return null;
        });

        //19合同处理成分结果数据
        tasks.add(() -> {
            plistQdToLineupResultCheDO = subService.getQdToLineupResultChe();
//            logger.info("******19合同处理成分结果数据获取完毕*******");
            return null;
        });

        //20合同处理坯料性能要求
        tasks.add(() -> {
            plistQdToLineupResultSlabphyDO = subService.getQdToLineupResultSlabphy();
//            logger.info("******20合同处理坯料性能要求数据获取完毕*******");
            return null;
        });

        //21炉次工序成分代表实绩表
        tasks.add(() -> {
            plistQeTqHeatcheStandResultDO = subService.getQeTqHeatcheStandResult();
//            logger.info("******21炉次工序成分代表实绩表数据获取完毕*******");
            return null;
        });

        //22炉次待委托表
        tasks.add(() -> {
            plistQeTqPonoWaitTestDO = subService.getQeTqPonoWaitTest();
//            logger.info("******22炉次待委托表数据获取完毕*******");
            return null;
        });

        //23坯料低倍委托实绩表
        tasks.add(() -> {
            plistQeTqSlabMacroResultDO = subService.getQeTqSlabMacroResult();
//            logger.info("******23坯料低倍委托实绩表数据获取完毕*******");
            return null;
        });

        //24连铸板坯生产规格基准表
        tasks.add(() -> {
            plistPmSlabSpecBtDO = subService.getPmSlabSpecBt();
//            logger.info("******23连铸板坯生产规格基准表数据获取完毕*******");
            return null;
        });

        // 执行所有任务
        try {
            executorService.invokeAll(tasks);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        logger.info("1:板坯数据：" + plistSlab.size());
        logger.info("2:订单数据：" + plistOrder.size());
        logger.info("3:生产合同产品属性表：" + plistPmProContProdAttrDO.size());
        logger.info("4:合同产线工艺参数表：" + plistPmContMscTechDO.size());
        logger.info("5:组板设计规则集：" + plistPmPdRuleSetDO.size());
        logger.info("6:组板设计规则集详细：" + plistPmPdRuleItemDO.size());
        logger.info("7:最小轧制厚度公差范围基准表：" + plistPmMinThickTolBtDO.size());
        logger.info("8:钢板轧制规格对应收得率基准表：" + plistPmPlateSpecYieldBtDO.size());
        logger.info("9:强度级别对照基准表：" + plistPmStCodeBtDO.size());
        logger.info("10:粗切位置决定基准表：" + plistPmCutPlaceBtDO.size());
        logger.info("11:组板常数基准表：" + plistPmPdConstBtDO.size());
        logger.info("12:钢板切缝切头尾范围基准表：" + plistPmPlateCutLenBtDO.size());
        logger.info("13:组板临时收得率基准表：" + plistPmPdTmpYieldBtDO.size());
        logger.info("14:钢板切边量基准表：" + plistPmPlateTrimBtDO.size());
        logger.info("15:余材组板内部钢种以优充次对照表：" + plistPmSlabReplBtDO.size());
        logger.info("16:热处理炉基准规格表：" + plistPmHeatTreatFurnaceBtDO.size());
        logger.info("17:特殊钢种余量增加表：" + plistPmSpeStAddvBtDO.size());
        logger.info("18:合同处理成分结果数据：" + plistQdToLineupResultCheDO.size());
        logger.info("19:合同处理坯料性能要求：" + plistQdToLineupResultSlabphyDO.size());
        logger.info("20:炉次工序成分代表实绩表：" + plistQeTqHeatcheStandResultDO.size());
        logger.info("21:炉次待委托表：" + plistQeTqPonoWaitTestDO.size());
        logger.info("22:坯料低倍委托实绩表：" + plistQeTqSlabMacroResultDO.size());
        logger.info("23:连铸板坯生产规格基准表：" + plistPmSlabSpecBtDO.size());

        // 关闭线程池
        executorService.shutdown();

        //endregion  多线程获取基础数据

    }

    /**
     * 数据预处理
     */
    public void SetPreData() {
        //订单数据处理
        SetOrderLackData();

        //订单根据尺寸初次分组
        SetGroupOrderFir();

    }

    /**
     * 二次分组根据板坯性能删选分组
     */
    public void SetGrpDataBySlab(SlabInfoDO SInfo) {

        gltGrpOrderFir.parallelStream().forEach(e -> {

            // gltGCalGrp 产生可计算分组

            //压缩比  Compression ratio

            //宽展比 Wide ratio

            //钢种标准对比

            GCalGrp GCG = new GCalGrp();
            GCG.setLtInfoGPreOrder(new ArrayList<>());
            e.getLtInfoGPreOrder().parallelStream().forEach(f -> {

                try {
                    //化学成分  chemical composition
                    if (!ChkChemical(SInfo, f)) {
                        return;
                    }
                } catch (Exception e1) {
                    log.error("化学成分", e1);
                    return;
                }
                try {
                    //低倍检测  low-power check
                    if (!ChkLowPower(SInfo, f)) {
                        return;
                    }
                } catch (Exception e1) {
                    log.error("低倍检测", e1);
                    return;
                }

                //GCG.setLtInfoGPreOrder(new ArrayList<>());
                GCG.getLtInfoGPreOrder().add(f);//添加订单信息
            });

            //存在订单，添加可计算分组
            if (GCG.getLtInfoGPreOrder().size() > 0) {

                //GCG.getLtInfoGPreOrder().stream().filter(e->e.)
                //最大宽度
                OptionalInt maxWidth = GCG.getLtInfoGPreOrder().stream()
                        .mapToInt(GPreOrder::getWid)
                        .max();
                //最大宽度余量
                OptionalInt maxAddWidth = GCG.getLtInfoGPreOrder().stream()
                        .mapToInt(GPreOrder::getWidthAddv)
                        .max();

                //宽展比   保留2位小数
                BigDecimal decWidRate = BigDecimal.valueOf(maxWidth.orElse(0)).divide(SInfo.getSlabActWid(), 2, RoundingMode.HALF_UP);

                //最大厚度公差最小值
                Optional<BigDecimal> maxTolThkMin = GCG.getLtInfoGPreOrder().stream()
                        .map(GPreOrder::getThickTolMin)
                        .reduce(BigDecimal::max);

                //轧制厚度   厚度+厚度公差下限最大值
                BigDecimal decThk = e.getThk().add(maxTolThkMin.orElse(BigDecimal.ZERO));

                //轧制宽度   最大宽度+宽度余量
                int intWid = maxWidth.orElse(0) + maxAddWidth.orElse(0);

                //切边
                int intTrimWid = 0;
                //切边量
                if (plistPmPlateTrimBtDO.stream().anyMatch(p -> p.getMscLineNo().equals("1030")
                        && p.getPlateThickFrom().compareTo(decThk) <= 0
                        && p.getPlateThickTo().compareTo(decThk) >= 0
                        && p.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) <= 0
                        && p.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) >= 0
                        && p.getWidenFrom().compareTo(decWidRate) <= 0
                        && p.getWidenTo().compareTo(decWidRate) >= 0
                )) {

                    intTrimWid = plistPmPlateTrimBtDO.stream().filter(p -> p.getMscLineNo().equals("1030")
                            && p.getPlateThickFrom().compareTo(decThk) <= 0
                            && p.getPlateThickTo().compareTo(decThk) >= 0
                            && p.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) <= 0
                            && p.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) >= 0
                            && p.getWidenFrom().compareTo(decWidRate) <= 0
                            && p.getWidenTo().compareTo(decWidRate) >= 0
                    ).collect(Collectors.toList()).get(0).getTrimWidth().intValue();
                }
                //轧制宽度= 最大宽度 + 最大宽度余量 + 切边量
                intWid = intWid + intTrimWid;
                //宽度
                GCG.setWid(intWid);

                //厚度
                GCG.setThk(decThk);

                //切边
                GCG.setWid_Add(intTrimWid);

                gltGCalGrp.add(GCG);
            }

        });

    }

    //region  验证方法

    /**
     * 化学成分判断
     */
    public boolean ChkChemical(SlabInfoDO SInfo, GPreOrder GrpOrd) {

        //        - 炉次工序成分代表实绩表QE_TQ_HEATCHE_STAND_RESULT      =>  制造命令号PONO
        //        - 合同处理成分结果数据表QD_TO_LINEUP_RESULT_CHE  =>合同号+成分类型CHE_TYPE=RX

        //炉次化学成分

        if (!plistQeTqHeatcheStandResultDO.stream().anyMatch(p -> p.getPono().equals(SInfo.getPono()))) {
            //ltQthsr=plistQeTqHeatcheStandResultDO.stream().filter(p->p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());
            return false;
        }

        //订单要求化学成分
        // logger.info("化学成分订单号：" + GrpOrd.getOrderNo());
        List<QdToLineupResultCheDO> ltLrc = new ArrayList<>();
        int intOrd = plistQdToLineupResultCheDO.stream().filter(p -> p.getOrderNo().equals(null)).collect(Collectors.toList()).size();
        int intRX = plistQdToLineupResultCheDO.stream().filter(p -> p.getCheType().equals(null)).collect(Collectors.toList()).size();
        if (intOrd > 0 || intRX > 0) {
            logger.info("*************************化学成分订单号：{}-{}-{}", GrpOrd.getOrderNo(), intOrd, intRX);
        }
        if (plistQdToLineupResultCheDO == null) {
            logger.info("*************************化学成分订单号：{}-{}-{}", GrpOrd.getOrderNo(), intOrd, intRX);
        }
        if (plistQdToLineupResultCheDO.stream().anyMatch(p -> p.getOrderNo().equals(GrpOrd.getOrderNo()) && p.getCheType().equals("RX"))) {
            ltLrc = plistQdToLineupResultCheDO.stream().filter(p -> p.getOrderNo().equals(SInfo.getOrderNo()) && p.getCheType().equals("RX")).collect(Collectors.toList());
        }

        //信息不存在返回
        if (ltLrc.size() == 0) {
            return false;
        }

        List<QeTqHeatcheStandResultDO> ltQthsr = plistQeTqHeatcheStandResultDO.stream().filter(p -> p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());

        //判断合同的每个化学成分是否满足条件
        boolean blnChk = ltLrc.stream().anyMatch(p -> {
            if (!ltQthsr.stream().anyMatch(e -> e.getElm().equals(p.getElm())
                    && (e.getTestOriginalResult().compareTo(NumberUtils.createBigDecimal(p.getElmMin())) >= 0
                    && e.getTestOriginalResult().compareTo(NumberUtils.createBigDecimal(p.getElmMax())) <= 0))) {
                return false;
            }
            return true;
        });

        if (!blnChk) {
            return false;
        }
        return true;
    }

    /**
     * 低倍检测
     */
    public boolean ChkLowPower(SlabInfoDO SInfo, GPreOrder GrpOrd) {

        // - 委托  QE_TQ_PONO_WAIT_TEST => 制造命令号PONO
        // - 坯料低倍委托实绩表QE_TQ_SLAB_MACRO_RESULT => 制造命令号PONO
        // - 合同处理坯料性能要求表QD_TO_LINEUP_RESULT_SLABPHY = > 合同号

        //  1、先通过制造命令号PONO查询QE_TQ_PONO_WAIT_TEST是否存在委托原因(该字段为多选字段内容)TEST_REASON=1的数据
        //    ，如未查询出数据则RESULT=1，如存在则通过制造命令号PONO抓取坯料低倍委托实绩表QE_TQ_SLAB_MACRO_RESULT
        //    ，获取实绩低倍数据，遍历合同号，再通过合同号查询合同处理坯料性能要求表QD_TO_LINEUP_RESULT_SLABPHY，获取该合同下低倍数据。
        //  2、对查出的数据进行对比，例如：合同中心偏析A字段等于1表示有要求，如中心偏析A实绩无数据则合同校验结果为不合格
        //    ，如对应实绩有数据则判断合同中心偏析A上限字段是否大于中心偏析A实绩，如大于则这个合同中心偏析A子项目合格
        //    ，如小于则合同校验结果为不合格，如合同中心偏析A字段等于0表示无要求，则不进行判断，依次判断到等轴晶率
        //    ，判断到合同其他缺陷A时需要特殊处理，如其他缺陷A等于1，之后拿合同其他缺陷A类型字段与实绩的其他缺陷A、B、C、D、E进行对比
        //    ，判断是否有相同的值，如无则合同校验结果为不合格，有则拿对应的其他缺陷A、B、C、D、E实绩与合同其他缺陷A上限进行比较
        //    ，如合同其他缺陷A上限大于其他缺陷A、B、C、D、E实绩，则这个合同其他缺陷A子项目合格，如小于则合同校验结果为不合格
        //    ，依次判断合同其他缺陷A、B、C、D、E，判定到合同校验结果为不合格时可以终止此层的循环，记录对应数据
        //    ，进入下一个合同号的循环，合格的数据也需要记录。

        //低倍合同要求表
        List<QdToLineupResultSlabphyDO> ltQthsr = plistQdToLineupResultSlabphyDO.stream().filter(p -> p.getOrderNo().equals(GrpOrd.getOrderNo())).collect(Collectors.toList());

        //判断合同是否需要判断低倍
        if (ltQthsr.size() == 0) {
            return true;
        }

        //委托  是否存在委托
        if (!plistQeTqPonoWaitTestDO.stream().anyMatch(p -> p.getPono().equals(SInfo.getPono()) && p.getTestReason().equals("1"))) {
            //ltQthsr=plistQeTqHeatcheStandResultDO.stream().filter(p->p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());
            return false;
        }

        //坯料低倍委托实绩表
        List<QeTqSlabMacroResultDO> ltSmr = new ArrayList<>();
        if (plistQeTqSlabMacroResultDO.stream().anyMatch(p -> p.getPono().equals(SInfo.getPono()))) {
            ltSmr = plistQeTqSlabMacroResultDO.stream().filter(p -> p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());
        }

        //信息不存在返回
        if (ltSmr.size() == 0) {
            return false;
        }
        //实例一个新的结果数据
        List<QeTqSlabMacroResultDO> ltSmrReslt = ltSmr;

        //中心偏析A
        if (ltQthsr.stream().anyMatch(p -> p.getCenterSgrgA().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterSgrgAResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterSgrgAMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterSgrgAResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }

        //        中心偏析B
        if (ltQthsr.stream().anyMatch(p -> p.getCenterSgrgB().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterSgrgBResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterSgrgBMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterSgrgBResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        中心偏析C
        if (ltQthsr.stream().anyMatch(p -> p.getCenterSgrgC().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterSgrgCResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterSgrgCMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterSgrgCResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        中心疏松
        if (ltQthsr.stream().anyMatch(p -> p.getCenterPorosity().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterPorosityResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterPorosityMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterPorosityResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        白亮带
        if (ltQthsr.stream().anyMatch(p -> p.getWhiteBand().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getWhiteBandResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getWhiteBandMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getWhiteBandResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        中间裂纹
        if (ltQthsr.stream().anyMatch(p -> p.getCrackCenter().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCrackCenterResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCrackCenterMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCrackCenterResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        三角区裂纹级别
        if (ltQthsr.stream().anyMatch(p -> p.getTriCrackGrade().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getTriCrackGradeResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getTriCrackGradeMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getTriCrackGradeResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        角裂级别
        if (ltQthsr.stream().anyMatch(p -> p.getAngleCrackGrade().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getAngleCrackGradeResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getAngleCrackGradeMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getAngleCrackGradeResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        三氧化二铝夹杂
        if (ltQthsr.stream().anyMatch(p -> p.getInclu().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getIncluResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getIncluMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getIncluResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        缩孔
        if (ltQthsr.stream().anyMatch(p -> p.getHore().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getHoreResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getHoreMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getHoreResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        蜂窝状气泡
        if (ltQthsr.stream().anyMatch(p -> p.getCellularBubble().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCellularBubbleResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCellularBubbleMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCellularBubbleResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        等轴晶率
        if (ltQthsr.stream().anyMatch(p -> p.getWafer().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getWaferResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getWaferMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getWaferResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷A
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectA().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectAResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectAMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectAResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷B
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectB().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectBResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectBMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectBResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷C
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectC().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectCResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectCMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectCResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷D
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectD().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectDResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectDMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectDResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷E
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectE().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectEResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectEMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectEResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * 压缩比判断
     * <p>
     * 本方法用于检查给定的板坯信息和订单信息之间的压缩比是否满足要求
     * 压缩比是通过比较订单的厚度和板坯的实际厚度计算得出的
     *
     * @param SInfo  板坯信息对象，包含板坯的实际厚度等信息
     * @param GrpOrd 订单信息对象，包含订单的厚度和压缩比下限等信息
     * @return 如果压缩比满足要求，则返回true；否则返回false
     */
    public boolean ChkCompress(SlabInfoDO SInfo, GPreOrder GrpOrd) {
        try {
            //判断GGrpOrderFir的压缩比下限
            BigDecimal decCompressRate = new BigDecimal(0);

            //SInfo的厚度除以GrpOrd的厚度等于压缩比decCompressRate
            decCompressRate = GrpOrd.getThk().divide(SInfo.getSlabActThk(), 2, BigDecimal.ROUND_HALF_UP);

            //压下比大于产线要求的下限值
            if (decCompressRate.compareTo(GrpOrd.getInfoPmContMscTechDO().getCompressMin()) < 0) {
                return false;
            } else {
                return true;
            }
        } catch (Exception e) {
            logger.error("压缩比验证出错", e);
            return false;
        }

    }

    /**
     * 宽展比判断
     */
    public boolean ChkWid(SlabInfoDO SInfo, GPreOrder GrpOrd) {


        return true;
    }

    //endregion

    /**
     * 根据缺乏数量创建相应数量的订单对象，并设置其属性
     * 此方法是同步的，以确保在多线程环境中对gltOrder的线程安全访问
     *
     * @param prPmProContDO 主合同信息对象，包含订单的基本信息和缺乏数量
     * @param ppcpa         订单产品属性对象，包含订单产品的详细属性
     * @param pcmt          订单次要技术信息对象，包含订单的次要技术要求
     */
    public static synchronized void SetAllOrder(PmProContDO prPmProContDO, PmProContProdAttrDO ppcpa, PmContMscTechDO pcmt) {

        // 获取缺乏的数量
        BigDecimal lackNum = prPmProContDO.getPdLackQty();
        // 如果缺乏数量大于0，则根据缺乏数量创建相应数量的订单对象
        if (lackNum.intValue() > 0) {
            for (int i = 0; i < lackNum.intValue(); i++) {
                // 创建一个新的GPreOrder对象
                GPreOrder GPO = new GPreOrder();
                // 设置订单号，自增确保唯一性
                GPO.setIDValue(intOrdNum++);
                // 设置厚度
                GPO.setThk(ppcpa.getOrderThick());
                // 设置宽度
                GPO.setWid(ppcpa.getOrderWidth().intValue());
                // 设置宽度余量
                GPO.setWidthAddv(ppcpa.getWidthAddv().intValue());
                // 设置长度
                GPO.setLen(ppcpa.getOrderLen());
                // 设置厚度公差下限
                GPO.setThickTolMin(ppcpa.getThickTolMin());
                // 设置订单号
                GPO.setOrderNo(prPmProContDO.getOrderNo());
                // 关联订单信息对象
                GPO.setInfoPmProContDo(prPmProContDO);
                // 关联订单产品属性对象
                GPO.setInfoPmProContProdAttrDO(ppcpa);
                // 关联订单次要技术信息对象
                GPO.setInfoPmContMscTechDO(pcmt);

                // 将创建的订单对象添加到全局订单列表中
                gltOrder.add(GPO);
            }
        }
    }

    /**
     * 订单数据欠量处理
     */
    public void SetOrderLackData() {

        //TODO:箭头函数订单多线程处理数据，升序，降序，重组。
        //使用 Stream.parallel() 来并行执行 for 循环  OrderNo  升序   PdLackQty 降序
        plistOrder.parallelStream()
                .sorted(Comparator.comparing(PmProContDO::getOrderNo)
                        .thenComparing(Comparator.comparing(PmProContDO::getPdLackQty).reversed()))
                .forEach(i -> {
                    PmProContProdAttrDO ppcpa = new PmProContProdAttrDO();
                    PmContMscTechDO pcmt = new PmContMscTechDO();

                    try {
                        if (plistPmProContProdAttrDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo())).count() > 0) {
                            ppcpa = plistPmProContProdAttrDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo())).findFirst().orElseThrow(() -> new NoSuchElementException("No PmProContProdAttrDO found with name Charlie " + i.getOrderNo()));

                            if (plistPmContMscTechDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo()) && p.getMscLineNo().equals("1030")).count() > 0) {
                                // 可能抛出异常的代码
                                pcmt = plistPmContMscTechDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo())).findFirst().orElseThrow(() -> new NoSuchElementException("No PmContMscTechDO found with name Charlie  " + i.getOrderNo()));
                                SetAllOrder(i, ppcpa, pcmt);
                            } else {
//                                logger.info("缺少产线信息：" + i.getOrderNo());
                            }
                        } else {
//                            logger.info("缺少属性信息：" + i.getOrderNo());
                        }

                    } catch (NullPointerException ex1) {
                        System.out.println("错误信息：" + ex1.getMessage());
                    }
                });

        //        //排序后产生新的List
        //      List<PmProContDO>  ltNewProContDO =   plistOrder.parallelStream()
        //                .sorted(Comparator.comparing(PmProContDO ::getOrderNo)
        //                        .thenComparing(Comparator.comparing(PmProContDO::getPdLackQty).reversed()))
        //                .collect(Collectors.toList());

        System.out.println("订单欠量数：" + gltOrder.size());
    }

    /**
     * 根据尺寸和钢种对订单进行初次分组
     * 该方法通过并行流处理订单列表，根据订单的宽度、厚度和钢种标准进行分组
     * 分组结果存储在gltGrpOrderFir列表中，每个分组内的订单共享相同的尺寸和钢种特性
     */
    public void SetGroupOrderFir() {

        //region 多线程  数据流处理订单分组

        gltOrder.parallelStream().forEach(p -> {

            // 检查当前订单是否与已有的分组匹配
            if (gltGrpOrderFir.stream().anyMatch(e -> e.getWid().equals(p.getInfoPmProContProdAttrDO().getOrderWidth()) &&
                    e.getThk().equals(p.getInfoPmProContProdAttrDO().getOrderThick()) &&
                    e.getSG_STD().equals(p.getInfoPmProContProdAttrDO().getSgStd()))) {

                // 如果匹配成功，将订单添加到相应的分组中
                GGrpOrderFir GGOF = gltGrpOrderFir.stream().filter(e -> e.getWid().equals(p.getInfoPmProContProdAttrDO().getOrderWidth())
                        && e.getThk().equals(p.getInfoPmProContProdAttrDO().getOrderThick())
                        && e.getSG_STD().equals(p.getInfoPmProContProdAttrDO().getSgStd())
                ).findFirst().orElse(null);
                if (GGOF.getLtInfoGPreOrder() == null) {
                    GGOF.setLtInfoGPreOrder(new ArrayList<>());  // 确保列表已初始化
                }
                GGOF.getLtInfoGPreOrder().add(p);

            } else {

                // 如果没有匹配的分组，创建一个新的分组并添加该订单
                GGrpOrderFir GGOF = new GGrpOrderFir();
                GGOF.setIDValue(intGrpNum++);
                GGOF.setWid(p.getInfoPmProContProdAttrDO().getOrderWidth());
                GGOF.setThk(p.getInfoPmProContProdAttrDO().getOrderThick());
                //GGOF.setLen(p.getInfoPmProContProdAttrDO().getOrderLen());
                GGOF.setSG_STD(p.getInfoPmProContProdAttrDO().getSgStd());

                GGOF.setLtInfoGPreOrder(new ArrayList<>());  // 初始化列表
                GGOF.getLtInfoGPreOrder().add(p);
                gltGrpOrderFir.add(GGOF);
            }

        });

        //endregion

        // 输出初始分组数量
        logger.info("***初始分组数量：" + gltGrpOrderFir.size());

    }

    /**
     * 设置组计算结果
     * 根据提供的板坯信息和生产配置，计算出每个组的最大可轧制长度，并筛选出可行的订单组合
     *
     * @param SInfo 包含板坯实际长度、宽度、厚度等信息的SlabInfoDO对象
     */
    public void SetGroupCalRslt(SlabInfoDO SInfo) {

        //轧机最大长度
        int intRollMaxLen = plistPmPdConstBtDO.stream().filter(p -> p.getRollLineNo().equals("30")).findFirst().orElse(null).getHpmMaxPLen().intValue();
        //轧机最小长度
        int intRollMinLen = plistPmPdConstBtDO.stream().filter(p -> p.getRollLineNo().equals("30")).findFirst().orElse(null).getHpmMinPLen().intValue();

        //gltGCalGrp.parallelStream().forEach(e->{
        gltGCalGrp.parallelStream().forEach(e -> {
            //实例化计算结果数组
            GGrpCalRslt GGCR = new GGrpCalRslt();
            //可轧制最大长度
            int intMaxLen = 0;
            //轧制最大宽度
            int intWid = e.getWid();
            //轧制最大厚度
            BigDecimal intThk = e.getThk();
            //板坯长度
            int intSlabLen = SInfo.getSlabActLen().intValue();

            //判断可轧制最大长度 TODO：厚度计算Decimal取整计算，会有误差，后续判断误差值修正计算
            if ((SInfo.getSlabActLen().multiply(SInfo.getSlabActThk()).multiply(SInfo.getSlabActWid()).divide(intThk.multiply(BigDecimal.valueOf(intWid)), RoundingMode.HALF_UP)).compareTo(BigDecimal.valueOf(intRollMaxLen)) > 0) {
                intMaxLen = intRollMaxLen;
            } else {
                intMaxLen = (SInfo.getSlabActLen().multiply(SInfo.getSlabActThk()).multiply(SInfo.getSlabActWid()).divide(intThk.multiply(BigDecimal.valueOf(intWid)), RoundingMode.HALF_UP)).intValue();
            }
            //不能小于可轧制的最小长度
            if (intMaxLen < intRollMinLen) {
                return;
            }
            //轧制宽度
            GGCR.setCal_Wid(intWid);
            //轧制厚度
            GGCR.setCal_Thk(intThk);
            //订单厚度
            GGCR.setThk(e.getLtInfoGPreOrder().get(0).getThk());
            //最大长度
            GGCR.setCal_Len(intMaxLen);
            GGCR.setLtInfoGPreOrder(e.getLtInfoGPreOrder());

            gltGGrpCalRslt.add(GGCR);
        });

        if (gltGGrpCalRslt.size() == 0) {
            return;
        }
        //分组计算
        //gltGGrpCalRslt.parallelStream().forEach(p -> {
        gltGGrpCalRslt.parallelStream().forEach(p -> {
            // 测试数据
            //处理数据多处理
            List<Integer> ltValues = new ArrayList<>();
            List<Integer> ltweights = new ArrayList<>();
            List<String> ltOrder = new ArrayList<>();

            //logger.info( "订单总数量：" + p.getLtInfoGPreOrder().size());
            if (p == null) {
                return;
            }
            if (p.getLtInfoGPreOrder() == null) {
                return;
            }
            //计算可处理数据最大值
            p.getLtInfoGPreOrder().forEach(e -> {

                int intOrdCount = p.getCal_Len() / e.getLen();

                if (ltOrder.stream().filter(f -> f.equals(e.getOrderNo())).count() == intOrdCount) {
                    return;
                }
                ltOrder.add(e.getOrderNo());//添加订单号
                ltValues.add(e.getIDValue());//价值
                ltweights.add(e.getLen());//长度
            });

            // 生成 values 和 weights 数组
            long[] values = new long[ltValues.size()];
            long[][] weights = new long[1][ltweights.size()];

            for (int i = 0; i < ltValues.size(); i++) {
                values[i] = ltValues.get(i);       // 从订单中获取 value
                weights[0][i] = ltweights.get(i) + 5;   // 从订单中获取 weights
            }

            //是否展宽
            //String brnNeed="";
            //if (p.getCal_Wid()>SInfo.getSlabActWid().intValue()){
            //    brnNeed="1";//展宽
            //}else {
            //    brnNeed="0";//不展宽
            //}

            //TODO：轧制方向，轧制模式 没有其他值暂不考虑，之后数据完善后需要考虑
            PmPlateCutLenBtDO PPCLB = plistPmPlateCutLenBtDO.stream().
                    filter(f -> f.getMscLineNo().equals("1030")
                            && f.getBrrNeed().equals((p.getCal_Wid() > SInfo.getSlabActWid().intValue()) ? "1" : "0")
                            && f.getPlateThickFrom().compareTo(p.getCal_Thk()) <= 0
                            && f.getPlateThickTo().compareTo(p.getCal_Thk()) >= 0
                            && f.getSlabThickFrom().compareTo(SInfo.getSlabActThk()) <= 0
                            && f.getSlabThickTo().compareTo(SInfo.getSlabActThk()) >= 0
                            && f.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(p.getCal_Wid())) <= 0
                            && f.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(p.getCal_Wid())) >= 0
                    ).findFirst().orElse(null);

            if (PPCLB == null) {
                return;
            }

            //切缝长度
            int intPlateCutWidth = PPCLB.getPlateCutWidth().intValue();

            //切头
            int intHead = PPCLB.getHeadCut().intValue();
            //切尾
            int intTail = PPCLB.getTailCut().intValue();

            //去掉切头尾长度
            int intCap = p.getCal_Len() - intHead - intTail;

            long[] capacities = {intCap};

            //初始化方案
            Loader.loadNativeLibraries();

            //logger.info("values  Size:{}", p.getLtInfoGPreOrder().size());
            //for (int i = 0; i < ltweights.size(); i++) {
            //
            //    logger.info("weights:{}", ltweights.get(i));
            //
            //}

            // 调用 solve 方法来解决问题
            ArrayList<Long> ltInt = solve(values, weights, capacities);

            //实例化实际订单号
            p.setLtActInfoGPreOrder(new ArrayList<>());
            //循环返回的订单插入到实际数组中
            for (int i = 0; i < ltInt.size(); i++) {
                int indexi = i;
                p.getLtActInfoGPreOrder().add(p.getLtInfoGPreOrder().stream().filter(m -> m.getIDValue() == ltInt.get(indexi)).collect(Collectors.toList()).get(0));
            }

            //根据实际订单再计算
            //

            if (p.getLtActInfoGPreOrder().size() == 0) {
                return;
            }

            //最大宽度
            OptionalInt maxWidth = p.getLtActInfoGPreOrder().stream()
                    .mapToInt(GPreOrder::getWid)
                    .max();
            //最大宽度余量
            OptionalInt maxAddWidth = p.getLtActInfoGPreOrder().stream()
                    .mapToInt(GPreOrder::getWidthAddv)
                    .max();

            //宽展比   保留2位小数
            BigDecimal decWidRate = BigDecimal.valueOf(maxWidth.orElse(0)).divide(SInfo.getSlabActWid(), 2, RoundingMode.HALF_UP);

            //最大厚度公差最小值
            Optional<BigDecimal> maxTolThkMin = p.getLtActInfoGPreOrder().stream()
                    .map(GPreOrder::getThickTolMin)
                    .reduce(BigDecimal::max);

            //轧制厚度   厚度+厚度公差下限最大值
            BigDecimal decThk = p.getThk().add(maxTolThkMin.orElse(BigDecimal.ZERO));

            //轧制宽度   最大宽度+宽度余量
            int intWid = maxWidth.orElse(0) + maxAddWidth.orElse(0);

            //切边
            int intTrimWid = 0;
            //切边量
            if (plistPmPlateTrimBtDO.stream().anyMatch(f -> f.getMscLineNo().equals("1030")
                    && f.getPlateThickFrom().compareTo(decThk) <= 0
                    && f.getPlateThickTo().compareTo(decThk) >= 0
                    && f.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) <= 0
                    && f.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) >= 0
                    && f.getWidenFrom().compareTo(decWidRate) <= 0
                    && f.getWidenTo().compareTo(decWidRate) >= 0
            )) {
                intTrimWid = plistPmPlateTrimBtDO.stream().filter(f -> f.getMscLineNo().equals("1030")
                        && f.getPlateThickFrom().compareTo(decThk) <= 0
                        && f.getPlateThickTo().compareTo(decThk) >= 0
                        && f.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) <= 0
                        && f.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) >= 0
                        && f.getWidenFrom().compareTo(decWidRate) <= 0
                        && f.getWidenTo().compareTo(decWidRate) >= 0
                ).collect(Collectors.toList()).get(0).getTrimWidth().intValue();
            }
            //轧制宽度= 最大宽度 + 最大宽度余量 + 切边量
            intWid = intWid + intTrimWid;

            //厚度  获取p.getLtActInfoGPreOrder()中厚度公差最小值+厚度
            p.setCal_Thk(decThk);
            //宽度  获取p.getLtActInfoGPreOrder() 中宽度的最大值
            p.setCal_Wid(intWid);
            //宽度余量  获取p.getLtActInfoGPreOrder() 中宽度的最大值
            p.setWid_Add(maxAddWidth.orElse(0));
            //切边余量
            p.setTrimming(intTrimWid);
            //切缝
            p.setPlate_Cut_Wid(intPlateCutWidth);

            //订单合计长度+切缝   TODO：取样长度是否加入计算，后续再优化
            int intSumOrderLen = p.getLtActInfoGPreOrder().stream().mapToInt(GPreOrder::getLen).sum();
            int intSumPlateCutWid = p.getLtActInfoGPreOrder().size() * intPlateCutWidth;
            p.setEff_Len(intSumOrderLen + intSumPlateCutWid);

            PmPlateCutLenBtDO PPCLB_Act = plistPmPlateCutLenBtDO.stream().
                    filter(f -> f.getMscLineNo().equals("1030")
                            && f.getBrrNeed().equals((p.getCal_Wid() > SInfo.getSlabActWid().intValue()) ? "1" : "0")
                            && f.getPlateThickFrom().compareTo(p.getCal_Thk()) <= 0
                            && f.getPlateThickTo().compareTo(p.getCal_Thk()) >= 0
                            && f.getSlabThickFrom().compareTo(SInfo.getSlabActThk()) <= 0
                            && f.getSlabThickTo().compareTo(SInfo.getSlabActThk()) >= 0
                            && f.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(p.getCal_Wid())) <= 0
                            && f.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(p.getCal_Wid())) >= 0
                    ).findFirst().orElse(null);

            if (PPCLB_Act == null) {
                return;
            }

            //切头
            p.setHead_Len(PPCLB_Act.getHeadCut().intValue());
            //切尾
            p.setTail_Len(PPCLB_Act.getTailCut().intValue());
            //轧制长度
            p.setCal_Len(p.getEff_Len() + PPCLB_Act.getHeadCut().intValue() + PPCLB_Act.getTailCut().intValue());

            //板坯厚度
            p.setVirtualSlab_Thk(SInfo.getSlabActThk());
            //板坯宽度
            p.setVirtualSlab_Wid(SInfo.getSlabActWid().intValue());
            //板坯长度
            BigDecimal virtualSlabLen = p.getCal_Thk()
                    .multiply(BigDecimal.valueOf(p.getCal_Wid()))
                    .multiply(BigDecimal.valueOf(p.getCal_Len()))
                    .divide(SInfo.getSlabActThk().multiply(SInfo.getSlabActWid()), 0, RoundingMode.CEILING);
            p.setVirtualSlab_Len(virtualSlabLen.intValue());
            //板坯重量
            p.setVirtualSlab_Wgt(p.getVirtualSlab_Thk().multiply(BigDecimal.valueOf(p.getVirtualSlab_Len())).multiply(BigDecimal.valueOf(p.getVirtualSlab_Wid()) ).multiply(BigDecimal.valueOf(7.85)));

            //成材率计算
            BigDecimal rolledVolume = p.getCal_Thk()
                    .multiply(BigDecimal.valueOf(p.getCal_Wid()))
                    .multiply(BigDecimal.valueOf(p.getEff_Len()));

            BigDecimal slabVolume = SInfo.getSlabActThk()
                    .multiply(SInfo.getSlabActWid())
                    .multiply(BigDecimal.valueOf(p.getVirtualSlab_Len()));

            BigDecimal yieldRate = rolledVolume.divide(slabVolume, 2, RoundingMode.HALF_UP); // 保留4位小数

            // 设置成材率到 p 对象中
            p.setRate(yieldRate);

        });

        //判断结算结果长度和成材率是否可用
        gltGGrpCalRslt.parallelStream().forEach(p -> {

            //没有成材率
            if (p.getRate() == null) {
                p.setIntUseFlg(0);
                return;
            }


            //成材率大于90%
            if (p.getRate().compareTo(BigDecimal.valueOf(0.9)) <= 0) {
                p.setIntUseFlg(0);
                return;
            }
            int minWid = Math.min(p.getVirtualSlab_Wid(), p.getVirtualSlab_Len());
            int maxLen = Math.max(p.getVirtualSlab_Wid(), p.getVirtualSlab_Len());
            //	板坯厚度起止SLAB_THICK_FROM/TO包含余坯实际厚度;
            //	板坯宽度起止SLAB_WIDTH_FROM/TO包含余坯实际宽度;
            //	加热炉类型为‘C’，轧制产线号等于产线号后两位，连铸机类型等于1；
            if (plistPmSlabSpecBtDO.stream().filter(e -> e.getSlabThickFrom().compareTo(p.getVirtualSlab_Thk()) <= 0
                    && e.getSlabThickTo().compareTo(p.getVirtualSlab_Thk()) >= 0
                    && e.getSlabWidthFrom().compareTo(BigDecimal.valueOf(minWid)) <= 0
                    && e.getSlabWidthTo().compareTo(BigDecimal.valueOf(minWid)) >= 0
                    && e.getFurType().equals("C")
                    && e.getRollLineNo().equals(SInfo.getMscLineNo().substring(SInfo.getMscLineNo().length() - 2))
                    && e.getCcType().equals("1")
            ).count() > 0) {
                int intSlanLenMin = plistPmSlabSpecBtDO.stream().filter(e -> e.getSlabThickFrom().compareTo(p.getVirtualSlab_Thk()) <= 0
                        && e.getSlabThickTo().compareTo(p.getVirtualSlab_Thk()) >= 0
                        && e.getSlabWidthFrom().compareTo(BigDecimal.valueOf(minWid)) <= 0
                        && e.getSlabWidthTo().compareTo(BigDecimal.valueOf(minWid)) >= 0
                        && e.getFurType().equals("C")
                        && e.getRollLineNo().equals(SInfo.getMscLineNo().substring(SInfo.getMscLineNo().length() - 2))
                        && e.getCcType().equals("1")
                ).findFirst().orElse(null).getSlabMinLen().intValue();
                int intSlanLenMax = plistPmSlabSpecBtDO.stream().filter(e -> e.getSlabThickFrom().compareTo(p.getVirtualSlab_Thk()) <= 0
                        && e.getSlabThickTo().compareTo(p.getVirtualSlab_Thk()) >= 0
                        && e.getSlabWidthFrom().compareTo(BigDecimal.valueOf(minWid)) <= 0
                        && e.getSlabWidthTo().compareTo(BigDecimal.valueOf(minWid)) >= 0
                        && e.getFurType().equals("C")
                        && e.getRollLineNo().equals(SInfo.getMscLineNo().substring(SInfo.getMscLineNo().length() - 2))
                        && e.getCcType().equals("1")
                ).findFirst().orElse(null).getSlabMaxLen().intValue();

                //不能小于最小值
                if (maxLen <= intSlanLenMin) {
                    p.setIntUseFlg(0);
                    return;
                }

                //不能大于最大值
                if (maxLen >= intSlanLenMax) {
                    p.setIntUseFlg(0);
                    return;
                }

            } else {
                //数据不可用
                p.setIntUseFlg(0);
                return;
            }

            //数据可用
            p.setIntUseFlg(1);


        });

        //选择最优结果
        ChosBestRsl(SInfo);

        //递归
//        //小于最小入炉尺寸
//        if (SInfo.getRemLen().intValue() <2200)
//        {
//
//        }else{
//            SetGroupCalRslt(SInfo);
//        }
    }

    public void ChosBestRsl(SlabInfoDO SInfo) {

        // 筛选 UseFlg == 1 的数据
        List<GGrpCalRslt> filteredList = gltGGrpCalRslt.stream()
                .filter(g -> g.getIntUseFlg()== 1)
                .collect(Collectors.toList());

        // 找到重量最大成材率最大的数据
        Optional<GGrpCalRslt> maxWeightMaxYieldRate = filteredList.stream()
                .max(Comparator.comparing((GGrpCalRslt g) -> g.getVirtualSlab_Wgt())
                        .thenComparing(GGrpCalRslt::getRate));

        // 将筛选出的数据添加到 gltSelGGrpCalRslt 中
        maxWeightMaxYieldRate.ifPresent(gltSelGGrpCalRslt::add);

        if (maxWeightMaxYieldRate.isPresent()) {

            GGrpCalRslt maxRate = maxWeightMaxYieldRate.get();

            PmSubHisDO pmSubHisDO = new PmSubHisDO();
            pmSubHisDO.setSlabNo(SInfo.getSlabNo()); // 板坯号
            // 生成虚拟板坯号
            String virtualSlabNo = generateVirtualSlabNo(SInfo.getPono());
            pmSubHisDO.setVirtualSlabNo(virtualSlabNo); // 虚拟板坯号

            // 手动为每个字段赋值
            pmSubHisDO.setSlabThick(maxRate.getVirtualSlab_Thk()); // 厚度
            pmSubHisDO.setSlabWid(BigDecimal.valueOf(maxRate.getVirtualSlab_Wid())); // 宽度
            pmSubHisDO.setSlabLen(BigDecimal.valueOf(maxRate.getVirtualSlab_Len())); // 长度
            pmSubHisDO.setSlabWt(maxRate.getVirtualSlab_Wgt()); // 重量
            pmSubHisDO.setSubYield(maxRate.getRate()); // 替代率
            pmSubHisDO.setRelYield(maxRate.getRate()); // 利用率，假设没有对应字段，设置为默认值
            pmSubHisDO.setSubResult(""); // 处理结果，假设没有对应字段，设置为默认值
            pmSubHisDO.setSubInfo(""); // 处理内容，假设没有对应字段，设置为默认值
            pmSubHisDO.setProcessCode(""); // 进程代码，假设没有对应字段，设置为默认值
            pmSubHisDO.setFormPlateMode(""); // 组板方式，假设没有对应字段，设置为默认值
            pmSubHisDO.setStNo(""); // 内部钢种，假设没有对应字段，设置为默认值
            pmSubHisDO.setSubTime(new Date()); // 替代时间，假设没有对应字段，设置为当前时间
            pmSubHisDO.setCreationDate(new Date()); // 创建时间
            pmSubHisDO.setCreatedBy(BigDecimal.valueOf(781L)); // 创建人ID，假设没有对应字段，设置为默认值
            pmSubHisDO.setLastUpdatedBy(BigDecimal.valueOf(781L)); // 最近更新人ID，假设没有对应字段，设置为默认值
            pmSubHisDO.setLastUpdateDate(new Date()); // 最近更新时间
            pmSubHisDO.setTenantId(BigDecimal.ZERO); // 租户ID，假设没有对应字段，设置为默认值
            pmSubHisDO.setObjectVersionNumber(BigDecimal.ZERO); // 行版本号，用来处理锁，假设没有对应字段，设置为默认值
            pmSubHisDO.setAttribute1(""); // 假设没有对应字段，设置为默认值
            pmSubHisDO.setAttribute2(""); // 假设没有对应字段，设置为默认值
            pmSubHisDO.setAttribute3(""); // 假设没有对应字段，设置为默认值
            pmSubHisDO.setAttribute4(""); // 假设没有对应字段，设置为默认值
            pmSubHisDO.setAttribute5(""); // 假设没有对应字段，设置为默认值
            pmSubHisDO.setRollThk(maxRate.getVirtualSlab_Thk()); // 轧件厚
            pmSubHisDO.setRollWid(BigDecimal.valueOf(maxRate.getVirtualSlab_Wid())); // 轧件宽
            pmSubHisDO.setRollLen(BigDecimal.valueOf(maxRate.getVirtualSlab_Len())); // 轧件长
            pmSubHisDO.setHeadCut(BigDecimal.valueOf(maxRate.getHead_Len())); // 头部切舍量
            pmSubHisDO.setTailCut(BigDecimal.valueOf(maxRate.getTail_Len())); // 尾部切舍量
            pmSubHisDO.setWidCut(BigDecimal.valueOf(maxRate.getTrimming())); // 切边宽度，假设没有对应字段，设置为默认值
            pmSubHisDO.setSlitLen(BigDecimal.valueOf(maxRate.getPlate_Cut_Wid())); // 切缝长度
            pmSubHisDO.setNeedSlabLen(BigDecimal.valueOf(maxRate.getVirtualSlab_Len())); // 必要板坯长度
            pmSubHisDO.setNeedSlabWt(maxRate.getVirtualSlab_Wgt()); // 必要板坯重量

           plistpmSubHisDO.add(pmSubHisDO);
            //子表信息
            // 使用 AtomicInteger 获取索引
            AtomicInteger index = new AtomicInteger(0);
            AtomicInteger intX = new AtomicInteger(maxRate.getHead_Len());
            maxRate.getLtActInfoGPreOrder().forEach(
                    gPreOrder -> {
                        int currentIndex = index.getAndIncrement();

                        // 实例化 PmSubOrdDO 对象
                        PmSubOrdDO pmSubOrdDO = new PmSubOrdDO();

                        // 生成虚拟板坯号
                        pmSubOrdDO.setVirtualSlabNo(virtualSlabNo); // 虚拟板坯号

                        // 手动为每个字段赋值
                        pmSubOrdDO.setOrderNo(gPreOrder.getOrderNo()); // 合同号
                        pmSubOrdDO.setSgSign(gPreOrder.getInfoPmProContProdAttrDO().getSgSign()); // 牌号
                        pmSubOrdDO.setStNo(SInfo.getStNo()); // 内部钢种
                        pmSubOrdDO.setOrderThick(gPreOrder.getThk()); // 订货厚度
                        pmSubOrdDO.setOrderWidth(BigDecimal.valueOf(gPreOrder.getWid())); // 订货宽度
                        pmSubOrdDO.setLenAim(BigDecimal.valueOf(gPreOrder.getIntActLen())); // 订货长度

                        pmSubOrdDO.setPosX(BigDecimal.valueOf( intX.intValue())); // X坐标，假设默认为0
                        intX.addAndGet(gPreOrder.getLen()+5);//X坐标
                        pmSubOrdDO.setPosY(BigDecimal.valueOf(maxRate.getTrimming()+gPreOrder.getWid())); // Y坐标，假设默认为0

                        pmSubOrdDO.setDelivyDate(new Date()); // 交货日期，假设默认为当前时间
                        pmSubOrdDO.setAxisOrd(gPreOrder.getOrderNo()); // 轴合同
                        pmSubOrdDO.setLackNum(BigDecimal.ZERO); // 欠量，假设默认为0
                        pmSubOrdDO.setOverNum(BigDecimal.ZERO); // 超投，假设默认为0
                        pmSubOrdDO.setCreationDate(new Date()); // 创建时间
                        pmSubOrdDO.setCreatedBy(BigDecimal.valueOf(781L)); // 创建人ID，假设默认为781
                        pmSubOrdDO.setLastUpdatedBy(BigDecimal.valueOf(781L)); // 最近更新人ID，假设默认为781
                        pmSubOrdDO.setLastUpdateDate(new Date()); // 最近更新时间
                        pmSubOrdDO.setTenantId(BigDecimal.ZERO); // 租户ID，假设默认为0
                        pmSubOrdDO.setObjectVersionNumber(BigDecimal.ZERO); // 行版本号，假设默认为0
                        pmSubOrdDO.setAttribute1(""); // 假设没有对应字段，设置为默认值
                        pmSubOrdDO.setAttribute2(""); // 假设没有对应字段，设置为默认值
                        pmSubOrdDO.setAttribute3(""); // 假设没有对应字段，设置为默认值
                        pmSubOrdDO.setAttribute4(""); // 假设没有对应字段，设置为默认值
                        pmSubOrdDO.setAttribute5(""); // 假设没有对应字段，设置为默认值
                        pmSubOrdDO.setOrderSeqNo(String.valueOf(currentIndex)); // 合同顺序号，假设使用合同号作为顺序号

                        // 将 PmSubOrdDO 对象添加到列表中
                        plistpmSubOrdDO.add(pmSubOrdDO);

                    }
            );
        }

    }

    // 生成虚拟板坯号的方法
    private String generateVirtualSlabNo(String pono) {
        AtomicInteger counter = counterMap.computeIfAbsent(pono, k -> new AtomicInteger(0));
        int currentCounter = counter.incrementAndGet();
        return pono + "S9" + String.format("%03d", currentCounter);
    }

    //设置保存数据
    public void SetSaveDate(SlabInfoDO SInfo) {
        SetPmPdSlab(SInfo);//组板板坯表 命令板坯表 命令大板表
        SetPmPdPlateCut();//大板粗切位置表
        SetPmPdBlockPos();//组合子板位置表
        SetPmPdContSum();//组板合同总量表  命令参数表
        SetPmSmallPltPos();//组板小板位置表  命令小板表
//        SetPmCmdSlab();//命令板坯表
//        SetPmCmdLargePlt();//命令大板表
//        SetPmCmdCont();//命令参数表
//        SetPmCmdSmallPlt();//命令小板表
    }

    //组板板坯表
    public void SetPmPdSlab(SlabInfoDO SInfo) {
        for (int i = 0; i < gltGGrpCalRslt.size(); i++) {
            GGrpCalRslt ggcr = gltGGrpCalRslt.get(i);
            PmPdSlabDO pmPdSlabDO = new PmPdSlabDO();
            pmPdSlabDO.setPono(SInfo.getPono());
            pmPdSlabDO.setSlabDivNo(BigDecimal.ZERO); //TODO 板坯分割号 未赋值  查询板坯号等于slab_no的组板板坯表的记录数+1
            pmPdSlabDO.setVirtualSlabNo(ggcr.getVirtualSlabNo());
            pmPdSlabDO.setSlabNo(ggcr.getSlabNo());
            pmPdSlabDO.setMatDesignKind("S");
            pmPdSlabDO.setPdStatusHp("12");
            pmPdSlabDO.setPdResp("781"); // TODO 组板责任者 未赋值  操作人id
            pmPdSlabDO.setFormPlateMode("FormPlateMode"); //TODO 组板方式 未赋值
            pmPdSlabDO.setPickModeLenDir(BigDecimal.ZERO); //TODO 取板方式_长度方向 未赋值
            pmPdSlabDO.setPickModeWidDir(BigDecimal.ZERO); //TODO 取板方式_宽度方向 未赋值
            pmPdSlabDO.setFurType("C");// TODO 加热炉类型 暂时赋值为C
            pmPdSlabDO.setAutoManualFlag("M");
            pmPdSlabDO.setOrderRemainDiv("1");
            pmPdSlabDO.setCanDelFlag("1");
            List<String> orderNoList = ggcr.getLtActInfoGPreOrder().stream().map(GPreOrder::getOrderNo).distinct().collect(Collectors.toList());//根据实际合同信息 的合同号去重 取出板内合同个数
            pmPdSlabDO.setOrderNumInPlate(BigDecimal.valueOf(orderNoList.size()));
            pmPdSlabDO.setTotSmallPlateNum(BigDecimal.valueOf(ggcr.getLtActInfoGPreOrder().size()));

            Optional<PmContMscTechDO> resultPmContMscTechDO = plistPmContMscTechDO.stream()
                    .filter(item -> ggcr.getLtActInfoGPreOrder().get(0).getOrderNo().equals(item.getOrderNo())) //TODO 根据轴合同号查询合同工艺表  暂时取第一条合同
                    .findFirst();
            pmPdSlabDO.setAdTypeCode(resultPmContMscTechDO.get().getAdTypeCode());
            pmPdSlabDO.setAdModeCode(resultPmContMscTechDO.get().getAdModeCode());
            pmPdSlabDO.setCtrlRollCode(resultPmContMscTechDO.get().getCtrlRollCode());
            pmPdSlabDO.setCutModeCode(resultPmContMscTechDO.get().getCutModeCode());

            //TODO 缺少质量模块MES合同主档QD_TO_PO_ORDER
            pmPdSlabDO.setUltCode(resultPmContMscTechDO.get().getUltCode());//TODO 遍历组板合同，根据合同号从质量模块MES合同主档QD_TO_PO_ORDER读取ERP_UT_SPEC1探伤标准1、ERP_UT_SPEC2超声探伤标2、ERP_UT_SPEC3超声探伤标3，去重后所有数据拼接成一条记录

            String ultDetectMethodCode = "";
            String coolCode = "";
            // 遍历 组板合同列表
            for (GPreOrder gPreOrder : ggcr.getLtActInfoGPreOrder()) {
                String orderNo = gPreOrder.getOrderNo();
                String mscLineNo = gPreOrder.getInfoPmContMscTechDO().getMscLineNo();

                // 根据 orderNo 和 mscLineNo 在 plistPmContMscTechDO 中查找匹配项
                Optional<PmContMscTechDO> matchingItem = plistPmContMscTechDO.stream()
                        .filter(item -> orderNo.equals(item.getOrderNo()) && mscLineNo.equals(item.getMscLineNo()))
                        .findFirst();
                if (!ultDetectMethodCode.isEmpty()) {
                    if (!coolCode.isEmpty()) {
                        break;
                    } else {
                        coolCode = nvl(matchingItem.get().getCoolCode(), "");
                    }
                } else {
                    ultDetectMethodCode = nvl(matchingItem.get().getUltDetectMethodCode(), "");
                    if (coolCode.isEmpty()) {
                        coolCode = nvl(matchingItem.get().getCoolCode(), "");
                    }
                }
            }
            pmPdSlabDO.setUltDetectMethodCode(ultDetectMethodCode);
            pmPdSlabDO.setCoolCode(coolCode);

            Optional<PmProContProdAttrDO> resultPmProContProdAttrDO = plistPmProContProdAttrDO.stream()
                    .filter(item -> ggcr.getLtActInfoGPreOrder().get(0).getOrderNo().equals(item.getOrderNo())) //TODO 根据轴合同号查询生产合同产品属性表  暂时取第一条合同
                    .findFirst();
            pmPdSlabDO.setWithSideFlag(resultPmProContProdAttrDO.get().getProdSideMark());

            pmPdSlabDO.setHeadCut(BigDecimal.valueOf(ggcr.getHead_Len()));
            pmPdSlabDO.setTailCut(BigDecimal.valueOf(ggcr.getTail_Len()));

            String urgOrderFlag = "";
            for (GPreOrder gPreOrder : ggcr.getLtActInfoGPreOrder()) {
                String orderNo = gPreOrder.getOrderNo();

                // 根据 orderNo 和 mscLineNo 在 plistPmContMscTechDO 中查找匹配项
                Optional<PmProContProdAttrDO> pmProContProdAttrDOItem = plistPmProContProdAttrDO.stream()
                        .filter(item -> orderNo.equals(item.getOrderNo()))
                        .findFirst();
                urgOrderFlag = String.valueOf(nvl(pmProContProdAttrDOItem.get().getUrgOrderFlag(), ""));
                if (!urgOrderFlag.isEmpty()) {
                    break;
                }
            }
            pmPdSlabDO.setUrgOrderFlag(urgOrderFlag);

            pmPdSlabDO.setSgSign(SInfo.getSgSign());
            pmPdSlabDO.setStNo(SInfo.getStNo());
            pmPdSlabDO.setOutSgNo(SInfo.getStNo());

            pmPdSlabDO.setPreCleanSlabThick(SInfo.getSlabActThk());
            pmPdSlabDO.setPreCleanSlabWidth(SInfo.getSlabActWid());
            pmPdSlabDO.setPreCleanSlabLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setPreCleanSlabMaxLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setPreCleanSlabMinLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            BigDecimal weight = SInfo.getSlabActThk().multiply(SInfo.getSlabActWid()).multiply(BigDecimal.valueOf(ggcr.getVirtualSlab_Len())).multiply(BigDecimal.valueOf(7.85));
            pmPdSlabDO.setPreCleanSlabWt(weight);
            pmPdSlabDO.setPreCleanSlabMaxWt(weight);
            pmPdSlabDO.setPreCleanSlabMinWt(weight);

            pmPdSlabDO.setInfurSlabThick(SInfo.getSlabActThk());
            pmPdSlabDO.setInfurSlabWid(SInfo.getSlabActWid());
            pmPdSlabDO.setInfurSlabLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setInfurSlabMaxLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setInfurSlabMinLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setInfurSlabWt(weight);
            pmPdSlabDO.setInfurSlabMaxWt(weight);
            pmPdSlabDO.setInfurSlabMinWt(weight);

            pmPdSlabDO.setFixSlabLenFlag(0);
            pmPdSlabDO.setFixSlabLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));

            Optional<PmContMscTechDO> resultPmContMscTechDO1 = plistPmContMscTechDO.stream()
                    .filter(item -> ggcr.getLtActInfoGPreOrder().get(0).getOrderNo().equals(item.getOrderNo()) && ggcr.getLtActInfoGPreOrder().get(0).getInfoPmContMscTechDO().getMscLineNo().equals(item.getMscLineNo())) //TODO 根据轴合同号和产线号查询合同工艺表  暂时取第一条合同
                    .findFirst();

            pmPdSlabDO.setHpFlameCleanDiv(resultPmContMscTechDO1.get().getFlameCleanDiv());
            pmPdSlabDO.setSlabFinishCode(resultPmContMscTechDO1.get().getSlabFinishCode());

            Optional<PmPdSlabInfoDO> resultPmPdSlabInfoDO = plistPmPdSlabInfoDO.stream()
                    .filter(item -> ggcr.getSlabNo().equals(item.getSlabNo()))
                    .findFirst();
            pmPdSlabDO.setSlabPlaceCode(resultPmPdSlabInfoDO.get().getSlabPlaceCode());

            pmPdSlabDO.setRollAimThick(ggcr.getCal_Thk());
            pmPdSlabDO.setRollAimWidth(BigDecimal.valueOf(ggcr.getCal_Wid()));
            pmPdSlabDO.setRollAimLen(BigDecimal.valueOf(ggcr.getCal_Len()));
            pmPdSlabDO.setOrderThickAxis(ggcr.getLtActInfoGPreOrder().get(0).getThk());
            pmPdSlabDO.setCombinePlateWidth(BigDecimal.valueOf(ggcr.getLtActInfoGPreOrder().get(0).getWid()));
            pmPdSlabDO.setCombinePlateLen(BigDecimal.valueOf(ggcr.getLtActInfoGPreOrder().get(0).getLen()));
            pmPdSlabDO.setRollWtHp(weight);

            BigDecimal minThickTolMax = ggcr.getLtActInfoGPreOrder().stream()
                    .map(GPreOrder::getThickTolMax)
                    .min(BigDecimal::compareTo)
                    .orElse(BigDecimal.ZERO);
            pmPdSlabDO.setRollThickTolMax(minThickTolMax);
            BigDecimal maxThickTolMin = ggcr.getLtActInfoGPreOrder().stream()
                    .map(GPreOrder::getThickTolMin)
                    .max(BigDecimal::compareTo)
                    .orElse(BigDecimal.ZERO);
            pmPdSlabDO.setRollThickTolMin(maxThickTolMin);

            BigDecimal yield = (ggcr.getLtActInfoGPreOrder().stream()
                    .map(gPreOrder -> BigDecimal.valueOf(gPreOrder.getWid()).multiply(gPreOrder.getThk()).multiply(BigDecimal.valueOf(gPreOrder.getLen())))
                    .reduce(BigDecimal.ZERO, BigDecimal::add))
                    .multiply(BigDecimal.valueOf(7.85))
                    .divide(weight, 2, RoundingMode.HALF_UP);
            pmPdSlabDO.setCommandYield(yield);
            pmPdSlabDO.setNormYield(yield);
            pmPdSlabDO.setRollYield(yield);

            pmPdSlabDO.setEdgerUseFlag("Y");
            pmPdSlabDO.setBroadMasUseFlag("N");
            pmPdSlabDO.setFormMasUseFlag("N");
            pmPdSlabDO.setRollDirectCode("L"); //TODO 轧制方向代码 横轧C纵轧L  暂时默认L
            pmPdSlabDO.setSampleLenHead(ggcr.getSmp_Len());//TODO 头尾中 试样长度 暂时默认试样长度
            pmPdSlabDO.setSampleLenTail(ggcr.getSmp_Len());
            pmPdSlabDO.setSampleLenMid(ggcr.getSmp_Len());
            pmPdSlabDO.setRsCode("0");
            if (BigDecimal.valueOf(ggcr.getCal_Wid()).compareTo(SInfo.getSlabActWid()) > 0) {
                pmPdSlabDO.setBroadFlag("1");  //需要展宽
            } else {
                pmPdSlabDO.setBroadFlag("0");
            }

            pmPdSlabDO.setOrderNoAxis(ggcr.getLtActInfoGPreOrder().get(0).getOrderNo());//TODO 轴合同号 暂时取第一个订单号
            pmPdSlabDO.setMscLineNo("1030"); //TODO 产线号，暂时默认1030

            pmPdSlabDO.setTransFlag(BigDecimal.valueOf(0));

            pmPdSlabDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmPdSlabDO.setLastUpdatedBy(781L);
            pmPdSlabDO.setCreationDate(now);
            pmPdSlabDO.setLastUpdateDate(now);

            SetPmCmdSlab(pmPdSlabDO);//命令板坯表
            SetPmCmdLargePlt(pmPdSlabDO, ggcr);//命令大板表

            pmPdSlabDOList.add(pmPdSlabDO);
        }
    }

    //大板粗切位置表
    public void SetPmPdPlateCut() {
        for (int i = 0; i < gltGGrpCalRslt.size(); i++) {
            GGrpCalRslt ggcr = gltGGrpCalRslt.get(i);
            PmPdPlateCutDO pmPdPlateCutDO = new PmPdPlateCutDO();
            pmPdPlateCutDO.setPdMatNo(ggcr.getVirtualSlabNo());
            //TODO 粗切类型 粗切位置 位置序号 取值待定
//            pmPdPlateCutDO.setCutType();
//            pmPdPlateCutDO.setPositionType();
//            pmPdPlateCutDO.setPositionSeq();

            pmPdPlateCutDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmPdPlateCutDO.setLastUpdatedBy(781L);
            pmPdPlateCutDO.setCreationDate(now);
            pmPdPlateCutDO.setLastUpdateDate(now);
            pmPdPlateCutDOList.add(pmPdPlateCutDO);
        }
    }

    //组合子板位置表
    public void SetPmPdBlockPos() {
        for (int i = 0; i < gltGGrpCalRslt.size(); i++) {
            GGrpCalRslt ggcr = gltGGrpCalRslt.get(i);
            List<String> orderNoList = ggcr.getLtActInfoGPreOrder().stream()
                    .map(GPreOrder::getOrderNo).distinct().collect(Collectors.toList()); //合同号去重
            for (int j = 0; j < orderNoList.size(); j++) {
                PmPdBlockPosDO pmPdBlockPosDO = new PmPdBlockPosDO();
                pmPdBlockPosDO.setPdMatNo(ggcr.getVirtualSlabNo());
                pmPdBlockPosDO.setOrderNo(orderNoList.get(j));
                pmPdBlockPosDO.setOrderSeqNo(j + 1);//TODO 订单序号 暂时以实际订单信息列表顺序
                Integer blockPlateNum = ggcr.getLtActInfoGPreOrder().stream() // 当前合同号在实际合同列表中的条数
                        .filter(item -> item.getOrderNo().equals(pmPdBlockPosDO.getOrderNo()))
                        .collect(Collectors.toList())
                        .size();
                pmPdBlockPosDO.setBlockPlateNum(blockPlateNum);
                pmPdBlockPosDO.setSlabNo(ggcr.getSlabNo());
//                pmPdBlockPosDO.setOrderBlockNo(); //TODO 厚板组板块号 暂时不知如何赋值

                pmPdBlockPosDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
                pmPdBlockPosDO.setLastUpdatedBy(781L);
                pmPdBlockPosDO.setCreationDate(now);
                pmPdBlockPosDO.setLastUpdateDate(now);
                pmPdBlockPosDOList.add(pmPdBlockPosDO);
            }
        }
    }

    //组板合同总量表
    public void SetPmPdContSum() {
        for (int i = 0; i < gltGGrpCalRslt.size(); i++) {
            GGrpCalRslt ggcr = gltGGrpCalRslt.get(i);
            List<String> orderNoList = ggcr.getLtActInfoGPreOrder().stream()
                    .map(GPreOrder::getOrderNo).distinct().collect(Collectors.toList()); //合同号去重
            for (int j = 0; j < orderNoList.size(); j++) {
                PmPdContSumDO pmPdContSumDO = new PmPdContSumDO();

                pmPdContSumDO.setPdMatNo(ggcr.getVirtualSlabNo());
                pmPdContSumDO.setOrderNo(orderNoList.get(j));
                pmPdContSumDO.setOrderSeqNo(BigDecimal.valueOf(j + 1));
                List<GPreOrder> gPreOrders = ggcr.getLtActInfoGPreOrder().stream() // 当前合同号在实际合同列表中的条数
                        .filter(item -> item.getOrderNo().equals(pmPdContSumDO.getOrderNo()))
                        .collect(Collectors.toList());
                Integer orderPlateNum = gPreOrders.size();
                pmPdContSumDO.setOrderPlateNum(BigDecimal.valueOf(orderPlateNum));
                pmPdContSumDO.setSlabNo(ggcr.getSlabNo());
                pmPdContSumDO.setPdMatType("S");
                pmPdContSumDO.setPdStatus("12");
                pmPdContSumDO.setOrderThick(gPreOrders.get(0).getThk());
                pmPdContSumDO.setOrderWidth(BigDecimal.valueOf(gPreOrders.get(0).getWid()));
                pmPdContSumDO.setOrderLen(BigDecimal.valueOf(gPreOrders.get(0).getIntActLen()));

                pmPdContSumDO.setOrderLenMax(BigDecimal.valueOf(gPreOrders.get(0).getInfoPmProContProdAttrDO().getOrderMaxLen()));
                pmPdContSumDO.setOrderLenMin(BigDecimal.valueOf(gPreOrders.get(0).getInfoPmProContProdAttrDO().getOrderMinLen()));

                pmPdContSumDO.setSmallPlateThick(gPreOrders.get(0).getThk());
                pmPdContSumDO.setSmallPlateWidth(BigDecimal.valueOf(gPreOrders.get(0).getWid()));
                pmPdContSumDO.setSmallPlateLen(BigDecimal.valueOf(gPreOrders.get(0).getIntActLen()));//TODO 长度不确定用实际长度还是长度

                pmPdContSumDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
                pmPdContSumDO.setLastUpdatedBy(781L);
                pmPdContSumDO.setCreationDate(now);
                pmPdContSumDO.setLastUpdateDate(now);
                SetPmCmdCont(pmPdContSumDO, gPreOrders.get(0));//命令参数表
                pmPdContSumDOList.add(pmPdContSumDO);
            }
        }
    }

    //组板小板位置表
    public void SetPmSmallPltPos() {
        for (int i = 0; i < gltGGrpCalRslt.size(); i++) {
            GGrpCalRslt ggcr = gltGGrpCalRslt.get(i);
            for (int j = 0; j < ggcr.getLtActInfoGPreOrder().size(); j++) {
                GPreOrder gpo = ggcr.getLtActInfoGPreOrder().get(j);
                PmSmallPltPosDO pmSmallPltPosDO = new PmSmallPltPosDO();
                pmSmallPltPosDO.setPdMatNo(ggcr.getVirtualSlabNo());
                pmSmallPltPosDO.setOrderNo(gpo.getOrderNo());
                pmSmallPltPosDO.setOrderSeqNo(j + 1);
                pmSmallPltPosDO.setSlabNo(ggcr.getSlabNo());
                Optional<PmProContProdAttrDO> resultPmProContProdAttrDO = plistPmProContProdAttrDO.stream()
                        .filter(item -> gpo.getOrderNo().equals(item.getOrderNo())) //根据合同号查询生产合同产品属性表
                        .findFirst();
                if (resultPmProContProdAttrDO.get().getTrimCode().equals("0")) {
                    pmSmallPltPosDO.setSmallPlateDivNo("000");
                } else {
                    pmSmallPltPosDO.setSmallPlateDivNo(SMALL_PLATE_DIV_NO[j]);
                }
                pmSmallPltPosDO.setPickModeLenDir(0);//TODO 取板方式 长度方向 不知道怎么取暂时默认为0
                pmSmallPltPosDO.setPickModeWidDir(0);//TODO 取板方式 宽度方向 不知道怎么取暂时默认为0
                pmSmallPltPosDO.setOrderThick(gpo.getThk());
                pmSmallPltPosDO.setOrderWidth(BigDecimal.valueOf(gpo.getWid()));
                pmSmallPltPosDO.setOrderLen(gpo.getIntActLen()); //TODO 长度不确定用实际长度还是长度
                pmSmallPltPosDO.setOrderLenMax(gpo.getInfoPmProContProdAttrDO().getOrderMaxLen());
                pmSmallPltPosDO.setOrderLenMin(gpo.getInfoPmProContProdAttrDO().getOrderMinLen());

                pmSmallPltPosDO.setSmallPlateThick(gpo.getThk());
                pmSmallPltPosDO.setSmallPlateWidth(BigDecimal.valueOf(gpo.getWid()));
                pmSmallPltPosDO.setSmallPlateLen(gpo.getIntActLen()); //TODO 长度不确定用实际长度还是长度

                pmSmallPltPosDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
                pmSmallPltPosDO.setLastUpdatedBy(781L);
                pmSmallPltPosDO.setCreationDate(now);
                pmSmallPltPosDO.setLastUpdateDate(now);

                SetPmCmdSmallPlt(pmSmallPltPosDO);
                pmSmallPltPosDOList.add(pmSmallPltPosDO);
            }
        }
    }

    //命令板坯表
    public void SetPmCmdSlab(PmPdSlabDO pmPdSlabDO) {
        PmCmdSlabDO pmCmdSlabDO = new PmCmdSlabDO();
        pmCmdSlabDO.setPonoSlabNo(pmPdSlabDO.getVirtualSlabNo());
        pmCmdSlabDO.setTmpSlabNo(pmPdSlabDO.getVirtualSlabNo());
        pmCmdSlabDO.setSlabNo(pmPdSlabDO.getSlabNo());
        pmCmdSlabDO.setVirtualSlabNo(pmPdSlabDO.getVirtualSlabNo());
        pmCmdSlabDO.setPdStatusHp("18");
        pmCmdSlabDO.setShortSlabNumInLong(gltGGrpCalRslt.size());
        pmCmdSlabDO.setBigPlateNum(gltGGrpCalRslt.size());
        pmCmdSlabDO.setSlabDivNo(pmPdSlabDO.getSlabDivNo().intValue());
        pmCmdSlabDO.setOrderNumInPlate(pmPdSlabDO.getOrderNumInPlate().intValue());
        pmCmdSlabDO.setOrderRemainDiv(pmPdSlabDO.getOrderRemainDiv());
        pmCmdSlabDO.setMatDesignKind("S");
        pmCmdSlabDO.setProdCodeHp("");//TODO 厚板品种代码 暂时默认为空
        pmCmdSlabDO.setSgSign(pmCmdSlabDO.getSgSign());
        pmPdSlabDO.setStNo(pmPdSlabDO.getStNo());
        pmCmdSlabDO.setPreCleanSlabThick(pmPdSlabDO.getPreCleanSlabThick());
        pmCmdSlabDO.setPreCleanSlabWidth(pmPdSlabDO.getPreCleanSlabWidth());
        pmCmdSlabDO.setPreCleanSlabLen(pmPdSlabDO.getPreCleanSlabLen());
        pmCmdSlabDO.setPreCleanSlabMaxLen(pmPdSlabDO.getPreCleanSlabMaxLen());
        pmCmdSlabDO.setPreCleanSlabMinLen(pmPdSlabDO.getPreCleanSlabMinLen());
        pmCmdSlabDO.setPreCleanSlabWt(pmPdSlabDO.getPreCleanSlabWt());
        pmCmdSlabDO.setPreCleanSlabMaxWt(pmPdSlabDO.getPreCleanSlabMaxWt());
        pmCmdSlabDO.setPreCleanSlabMinWt(pmPdSlabDO.getPreCleanSlabMinWt());

        Optional<PmPdSlabInfoDO> resultPmPdSlabInfoDO = plistPmPdSlabInfoDO.stream()
                .filter(item -> pmPdSlabDO.getSlabNo().equals(item.getSlabNo()))
                .findFirst();
        pmCmdSlabDO.setMoldThick(resultPmPdSlabInfoDO.get().getMatActThick().intValue());
        pmCmdSlabDO.setMoldWid(resultPmPdSlabInfoDO.get().getMatActWidth().intValue());

        pmCmdSlabDO.setInfurSlabThick(pmPdSlabDO.getInfurSlabThick());
        pmCmdSlabDO.setInfurSlabWid(pmPdSlabDO.getInfurSlabWid());
        pmCmdSlabDO.setInfurSlabLen(pmPdSlabDO.getInfurSlabLen());
        pmCmdSlabDO.setInfurSlabMaxLen(pmPdSlabDO.getInfurSlabMaxLen());
        pmCmdSlabDO.setInfurSlabMinLen(pmPdSlabDO.getInfurSlabMinLen());
        pmCmdSlabDO.setInfurSlabWt(pmPdSlabDO.getInfurSlabWt());
        pmCmdSlabDO.setInfurSlabMaxWt(pmPdSlabDO.getInfurSlabMaxWt());
        pmCmdSlabDO.setInfurSlabMinWt(pmPdSlabDO.getInfurSlabMinWt());
        pmCmdSlabDO.setNeedSlabLen(pmPdSlabDO.getNeedSlabLen());
        pmCmdSlabDO.setNeedSlabWt(pmPdSlabDO.getNeedSlabWt());
        pmCmdSlabDO.setFixSlabLenFlag(String.valueOf(pmPdSlabDO.getFixSlabLenFlag()));
        pmCmdSlabDO.setFixSlabLen(pmPdSlabDO.getFixSlabLen().intValue());
        pmCmdSlabDO.setSlabPlaceCode(pmPdSlabDO.getSlabPlaceCode());
        pmCmdSlabDO.setHpFlameCleanDiv(pmPdSlabDO.getHpFlameCleanDiv());
        pmCmdSlabDO.setSlabFinishCode(pmPdSlabDO.getSlabFinishCode());
        pmCmdSlabDO.setCoolCode(pmPdSlabDO.getCoolCode());
        pmCmdSlabDO.setLastSlabFlag(String.valueOf(pmPdSlabDO.getLastSlabFlag()));
        pmCmdSlabDO.setAddSlabFlag(pmPdSlabDO.getAddSlabFlag());
        pmCmdSlabDO.setSlabPreProc1(""); //TODO  板坯预定通过工序1  文档中没有 有合同分割取值逻辑 暂时默认为空
        pmCmdSlabDO.setHotSendDiv("H");
        pmCmdSlabDO.setFurType(pmPdSlabDO.getFurType());
        pmCmdSlabDO.setHotChargeFlag("0");
        pmCmdSlabDO.setFormPlateTime(now); //TODO  组板时间 创建时间 暂时默认为当前时间
        pmCmdSlabDO.setPono(pmPdSlabDO.getPono());
        pmCmdSlabDO.setMscLineNo(pmPdSlabDO.getMscLineNo());
        pmCmdSlabDO.setIngotType(pmPdSlabDO.getIngotType());
        pmCmdSlabDO.setIgMatDesignKind(pmPdSlabDO.getIgMatDesignKind());
        pmCmdSlabDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
        pmCmdSlabDO.setLastUpdatedBy(781L);
        pmCmdSlabDO.setCreationDate(now);
        pmCmdSlabDO.setLastUpdateDate(now);
        pmCmdSlabDOList.add(pmCmdSlabDO);
    }

    //命令大板表
    public void SetPmCmdLargePlt(PmPdSlabDO pmPdSlabDO, GGrpCalRslt gGrpCalRslt) {
        PmCmdLargePltDO pmCmdLargePltDO = new PmCmdLargePltDO();
        pmCmdLargePltDO.setPonoSlabNo(pmPdSlabDO.getVirtualSlabNo());
        pmCmdLargePltDO.setTmpSlabNo(pmPdSlabDO.getVirtualSlabNo());
        pmCmdLargePltDO.setSlabNo(pmPdSlabDO.getSlabNo());
        pmCmdLargePltDO.setSlabDivNo(pmPdSlabDO.getSlabDivNo().intValue());
        pmCmdLargePltDO.setPdStatusHp("18");
        pmCmdLargePltDO.setFormPlateMode(pmPdSlabDO.getFormPlateMode());
        pmCmdLargePltDO.setPickModeLenDir(pmPdSlabDO.getPickModeLenDir().intValue());
        pmCmdLargePltDO.setPickModeWidDir(pmPdSlabDO.getPickModeWidDir().intValue());
        pmCmdLargePltDO.setFurType(pmPdSlabDO.getFurType());
        pmCmdLargePltDO.setAutoManualFlag(pmPdSlabDO.getAutoManualFlag());
        pmCmdLargePltDO.setOrderRemainDiv(pmPdSlabDO.getOrderRemainDiv());
        pmCmdLargePltDO.setMatDesignKind(pmPdSlabDO.getMatDesignKind());
        pmCmdLargePltDO.setOrderNumInPlate(pmPdSlabDO.getOrderNumInPlate().intValue());
        pmCmdLargePltDO.setTotSmallPlateNum(pmPdSlabDO.getTotSmallPlateNum().intValue());
        pmCmdLargePltDO.setProdCodeHp("");//TODO 厚板品种代码 暂时默认为空
        pmCmdLargePltDO.setAdTypeCode(pmPdSlabDO.getAdTypeCode());
        pmCmdLargePltDO.setAdModeCode(pmPdSlabDO.getAdModeCode());
        pmCmdLargePltDO.setUltCode(pmPdSlabDO.getUltCode());
        pmCmdLargePltDO.setCoolCode(pmPdSlabDO.getCoolCode());
        pmCmdLargePltDO.setCtrlRollCode(pmPdSlabDO.getCtrlRollCode());
        //TODO PM_PD_SLAB虚拟板坯号等于virtual_slab_no的PROD_SIDE_MARK，若为空或‘0’，则为‘C’，否则为PROD_SIDE_MARK  无PROD_SIDE_MARK这个字段
//        if(pmPdSlabDO.getProdSideMark() == null || pmPdSlabDO.getProdSideMark().equals("") || pmPdSlabDO.getProdSideMark().equals("0")){
//            pmCmdLargePltDO.setProdSideMark("C");
//        }else{
//            pmCmdLargePltDO.setProdSideMark(pmPdSlabDO.getProdSideMark());
//        }
        //TODO PM_PD_SLAB虚拟板坯号等于virtual_slab_no的PROD_SIDE_MARK，若为空或‘0’，则为‘C’，否则为PROD_SIDE_MARK  无PROD_SIDE_MARK这个字段
        pmCmdLargePltDO.setCutModeCode(pmPdSlabDO.getCutModeCode());
        pmCmdLargePltDO.setHeadCut(pmPdSlabDO.getHeadCut().intValue());
        pmCmdLargePltDO.setTailCut(pmPdSlabDO.getTailCut().intValue());
        pmCmdLargePltDO.setSgSign(gGrpCalRslt.getLtActInfoGPreOrder().get(0).getInfoPmProContProdAttrDO().getSgSign());//TODO 轴合同的标准牌号 暂时取第一条合同信息
        pmCmdLargePltDO.setSgSign(gGrpCalRslt.getLtActInfoGPreOrder().get(0).getInfoPmProContProdAttrDO().getSgSign());
        pmCmdLargePltDO.setStNo(pmPdSlabDO.getStNo());
        pmCmdLargePltDO.setTwoRollInterimFlag(String.valueOf(pmPdSlabDO.getTwoRollInterimFlag()));
        pmCmdLargePltDO.setRollAimThick(pmPdSlabDO.getRollAimThick());
        pmCmdLargePltDO.setRollAimWidth(pmPdSlabDO.getRollAimWidth());
        pmCmdLargePltDO.setRollAimLen(pmPdSlabDO.getRollAimLen());
        pmCmdLargePltDO.setRollThickTolMax(pmPdSlabDO.getRollThickTolMax());
        pmCmdLargePltDO.setRollThickTolMin(pmPdSlabDO.getRollThickTolMin());

        BigDecimal minThickTolMax = gGrpCalRslt.getLtActInfoGPreOrder().stream()
                .map(order -> order.getThickTolMax().add(order.getThk()))
                .min(BigDecimal::compareTo)
                .orElse(BigDecimal.ZERO);
        pmCmdLargePltDO.setMaxRollThick(minThickTolMax);
        BigDecimal maxThickTolMin = gGrpCalRslt.getLtActInfoGPreOrder().stream()
                .map(order -> order.getThickTolMin().add(order.getThk()))
                .max(BigDecimal::compareTo)
                .orElse(BigDecimal.ZERO);
        pmCmdLargePltDO.setMinRollThick(maxThickTolMin);

        pmCmdLargePltDO.setOrder1Thick(pmPdSlabDO.getOrderThickAxis());
        pmCmdLargePltDO.setCombinePlateWidth(pmPdSlabDO.getCombinePlateWidth());
        pmCmdLargePltDO.setCombinePlateLen(pmPdSlabDO.getCombinePlateLen());
        pmCmdLargePltDO.setRollWtHp(pmPdSlabDO.getRollWtHp());
        pmCmdLargePltDO.setNormYield(pmPdSlabDO.getNormYield());
        pmCmdLargePltDO.setCommandYield(pmPdSlabDO.getCommandYield());
        pmCmdLargePltDO.setRollYield(pmPdSlabDO.getRollYield());
        pmCmdLargePltDO.setEdgerUseFlag(pmPdSlabDO.getEdgerUseFlag());
        pmCmdLargePltDO.setBroadMasUseFlag(pmPdSlabDO.getBroadMasUseFlag());
        pmCmdLargePltDO.setFormMasUseFlag(pmPdSlabDO.getFormMasUseFlag());
        pmCmdLargePltDO.setRollDirectCode(pmPdSlabDO.getRollDirectCode());
        pmCmdLargePltDO.setSampleLenHead(pmPdSlabDO.getSampleLenHead());
        pmCmdLargePltDO.setSampleLenTail(pmPdSlabDO.getSampleLenTail());
        pmCmdLargePltDO.setSampleLenMid(pmPdSlabDO.getSampleLenMid());

        Optional<PmProContProdAttrDO> resultPmProContProdAttrDO = plistPmProContProdAttrDO.stream()
                .filter(item -> gGrpCalRslt.getLtActInfoGPreOrder().get(0).getOrderNo().equals(item.getOrderNo())) //TODO 根据轴合同号查询生产合同产品属性表  暂时取第一条合同
                .findFirst();
        pmCmdLargePltDO.setSampleLenGive(resultPmProContProdAttrDO.get().getSampleLenGive());
        pmCmdLargePltDO.setProdDensity(resultPmProContProdAttrDO.get().getProdDensity());
        pmCmdLargePltDO.setPlateDtCode(resultPmProContProdAttrDO.get().getProjectCode());

        pmCmdLargePltDO.setRsCode(pmPdSlabDO.getRsCode());
        pmCmdLargePltDO.setBroadFlag(pmPdSlabDO.getBroadFlag());
        pmCmdLargePltDO.setGFlag("");//TODO 气切可否 文档描述不清楚 暂时默认为空
        pmCmdLargePltDO.setCsFlag(""); //TODO pmPdSlabDO 无 CsFlag() 暂时默认为空
        pmCmdLargePltDO.setDsFlag(""); //TODO 若为G型组板，为‘0’；否则  取PM_PD_SLAB虚拟板坯号等于virtual_slab_no的同字段值 pmPdSlabDO 无 DsFlag() 暂时默认为空
        pmCmdLargePltDO.setDssFlag("");//TODO  若为G型组板，为‘0’；否则取PM_PD_SLAB虚拟板坯号等于virtual_slab_no的同字段值 pmPdSlabDO 无 DssFlag() 暂时默认为空
        pmCmdLargePltDO.setSsFlag("");//TODO 若为G型或A型组板，为‘0’；否则取PM_PD_SLAB虚拟板坯号等于virtual_slab_no的同字段值 pmPdSlabDO 无 SsFlag() 暂时默认为空
        pmCmdLargePltDO.setFormPlateTime(now);
        int count = gGrpCalRslt.getLtActInfoGPreOrder().size();
        pmCmdLargePltDO.setOtherPlateNo1(SMALL_PLATE_DIV_NO[count]);
        pmCmdLargePltDO.setOtherPlateNo2(SMALL_PLATE_DIV_NO[count + 1]);
        pmCmdLargePltDO.setOtherPlateNo3(SMALL_PLATE_DIV_NO[count + 2]);
        pmCmdLargePltDO.setOtherPlateNo4(SMALL_PLATE_DIV_NO[count + 3]);
        pmCmdLargePltDO.setMscLineNo(pmPdSlabDO.getMscLineNo());

        pmCmdLargePltDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
        pmCmdLargePltDO.setLastUpdatedBy(781L);
        pmCmdLargePltDO.setCreationDate(now);
        pmCmdLargePltDO.setLastUpdateDate(now);
        pmCmdLargePltDOList.add(pmCmdLargePltDO);
    }

    //命令参数表
    public void SetPmCmdCont(PmPdContSumDO pmPdContSumDO, GPreOrder gPreOrder) {
        PmCmdContDO pmCmdContDO = new PmCmdContDO();
        pmCmdContDO.setPonoSlabNo(pmPdContSumDO.getPdMatNo());
        pmCmdContDO.setTmpSlabNo(pmPdContSumDO.getPdMatNo());
        pmCmdContDO.setOrderNo(pmPdContSumDO.getOrderNo());
        pmCmdContDO.setOrderSeqNo(pmPdContSumDO.getOrderSeqNo().longValue());

        String orderNo = pmPdContSumDO.getOrderNo();
        String mscLineNo = gPreOrder.getInfoPmContMscTechDO().getMscLineNo();
        Optional<PmContMscTechDO> pmContMscTechDOItem = plistPmContMscTechDO.stream()
                .filter(item -> orderNo.equals(item.getOrderNo()) && mscLineNo.equals(item.getMscLineNo()))
                .findFirst();
        pmCmdContDO.setWholeBacklog(pmContMscTechDOItem.get().getWholeBacklog());
        pmCmdContDO.setStNo(pmContMscTechDOItem.get().getStNo());

        Optional<PmPdSlabDO> pmPdSlabDOItem = pmPdSlabDOList.stream()
                .filter(item -> pmPdContSumDO.getPdMatNo().equals(item.getVirtualSlabNo()))
                .findFirst();
        pmCmdContDO.setMatDesignKind(pmPdSlabDOItem.get().getMatDesignKind());
        pmCmdContDO.setSlabNo(pmPdSlabDOItem.get().getSlabNo());

        pmCmdContDO.setOrderTypeCode(gPreOrder.getInfoPmProContProdAttrDO().getOrderTypeCode());
        pmCmdContDO.setExportFlag(String.valueOf(gPreOrder.getInfoPmProContProdAttrDO().getExportFlag()));
        pmCmdContDO.setFinCustCode(gPreOrder.getInfoPmProContProdAttrDO().getFinCustCode());
        pmCmdContDO.setPsc(gPreOrder.getInfoPmProContProdAttrDO().getPsc());
        pmCmdContDO.setApn(gPreOrder.getInfoPmProContProdAttrDO().getApn());
        pmCmdContDO.setMsc(gPreOrder.getInfoPmProContProdAttrDO().getMsc());
        pmCmdContDO.setProdCodeHp(gPreOrder.getInfoPmProContProdAttrDO().getProdCodeHp());
        pmCmdContDO.setSgStd(gPreOrder.getInfoPmProContProdAttrDO().getSgStd());
        pmCmdContDO.setSmallPlateLen(pmPdContSumDO.getOrderPlateNum());
        pmCmdContDO.setProdSrMark(gPreOrder.getInfoPmProContProdAttrDO().getProdSrMark());
        pmCmdContDO.setProdSideMark(gPreOrder.getInfoPmProContProdAttrDO().getProdSideMark());
        pmCmdContDO.setProdShotMark(gPreOrder.getInfoPmProContProdAttrDO().getProdShotMark());
        pmCmdContDO.setProdNMark(gPreOrder.getInfoPmProContProdAttrDO().getProdNMark());
        pmCmdContDO.setProdQMark(gPreOrder.getInfoPmProContProdAttrDO().getProdQMark());
        pmCmdContDO.setProdTMark(gPreOrder.getInfoPmProContProdAttrDO().getProdTMark());
        pmCmdContDO.setProdAMark(gPreOrder.getInfoPmProContProdAttrDO().getProdAMark());
        pmCmdContDO.setBacklogUtMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogUtMark());
        pmCmdContDO.setBacklogQMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogQMark());
        pmCmdContDO.setBacklogTMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogTMark());
        pmCmdContDO.setBacklogAMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogAMark());
        pmCmdContDO.setBacklogNMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogNMark());
        pmCmdContDO.setBacklogDemagMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogDemagMark());
        pmCmdContDO.setBacklogPtMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogPtMark());
        pmCmdContDO.setBacklogMtMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogMtMark());
        pmCmdContDO.setBacklogTmcpMark(gPreOrder.getInfoPmProContProdAttrDO().getBacklogTmcpMark());
        pmCmdContDO.setUnitWtMax(gPreOrder.getInfoPmProContProdAttrDO().getOrderUnitMaxWt());
        pmCmdContDO.setUnitWtMin(gPreOrder.getInfoPmProContProdAttrDO().getOrderUnitMinWt());
        pmCmdContDO.setUnitWtAim(gPreOrder.getInfoPmProContProdAttrDO().getOrderUnitAimWt());
        pmCmdContDO.setOrderThick(gPreOrder.getInfoPmProContProdAttrDO().getOrderThick());
        pmCmdContDO.setOrderLen(BigDecimal.valueOf(gPreOrder.getInfoPmProContProdAttrDO().getOrderLen()));
        pmCmdContDO.setOrderWidth(gPreOrder.getInfoPmProContProdAttrDO().getOrderWidth());
        pmCmdContDO.setWidthDiv(String.valueOf(gPreOrder.getInfoPmProContProdAttrDO().getWidthDiv()));
        pmCmdContDO.setWidthMin(gPreOrder.getInfoPmProContProdAttrDO().getWidthMin());
        pmCmdContDO.setWidthMax(gPreOrder.getInfoPmProContProdAttrDO().getWidthMax());
        pmCmdContDO.setLenDiv(String.valueOf(gPreOrder.getInfoPmProContProdAttrDO().getLenDiv()));
        pmCmdContDO.setOrderLenMin(BigDecimal.valueOf(gPreOrder.getInfoPmProContProdAttrDO().getOrderMinLen()));
        pmCmdContDO.setOrderLenMax(BigDecimal.valueOf(gPreOrder.getInfoPmProContProdAttrDO().getOrderMaxLen()));
        pmCmdContDO.setSmallPlateThick(pmPdContSumDO.getSmallPlateThick());
        pmCmdContDO.setSmallPlateWidth(pmPdContSumDO.getSmallPlateWidth());
        pmCmdContDO.setSmallPlateLen(pmPdContSumDO.getSmallPlateLen());
        pmCmdContDO.setTrimCode(gPreOrder.getInfoPmProContProdAttrDO().getTrimCode());
        pmCmdContDO.setWidenMin(pmContMscTechDOItem.get().getWidenMin());
        pmCmdContDO.setCompressMin(pmContMscTechDOItem.get().getCompressMin());
        pmCmdContDO.setRollDirectCode(pmContMscTechDOItem.get().getRollDirectCode());
        pmCmdContDO.setCtrlRollCode(pmContMscTechDOItem.get().getCtrlRollCode());
        pmCmdContDO.setAdTypeCode(pmContMscTechDOItem.get().getAdTypeCode());
        pmCmdContDO.setAdModeCode(pmContMscTechDOItem.get().getAdModeCode());
        pmCmdContDO.setHotRectifyFlag(String.valueOf(pmContMscTechDOItem.get().getHotRectifyFlag()));
        pmCmdContDO.setHeadCut(pmPdSlabDOItem.get().getHeadCut().intValue());
        pmCmdContDO.setTailCut(pmPdSlabDOItem.get().getTailCut().intValue());
        pmCmdContDO.setCutModeCode(pmContMscTechDOItem.get().getAdModeCode());
        pmCmdContDO.setGFlag(String.valueOf(pmContMscTechDOItem.get().getGFlag()));
        pmCmdContDO.setCsFlag(String.valueOf(pmContMscTechDOItem.get().getCsFlag()));
        pmCmdContDO.setDssFlag(String.valueOf(pmContMscTechDOItem.get().getDssFlag()));
        pmCmdContDO.setDsFlag(String.valueOf(pmContMscTechDOItem.get().getDsFlag()));
        pmCmdContDO.setSsFlag(String.valueOf(pmContMscTechDOItem.get().getSsFlag()));
        pmCmdContDO.setHeatTreatMode(pmContMscTechDOItem.get().getHeatTreatMode());
        pmCmdContDO.setUltDetectMethodCode(pmContMscTechDOItem.get().getUltDetectMethodCode());
        pmCmdContDO.setUltCode("");//TODO 遍根据合同号从质量模块MES合同主档QD_TO_PO_ORDER读取超声探伤标准1、超声探伤标2、超声探伤标3、超声探伤标4、超声探伤标5，去重后所有数据用‘-’拼接成一条记录
        pmCmdContDO.setMscLineNo(mscLineNo);
        pmCmdContDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
        pmCmdContDO.setLastUpdatedBy(781L);
        pmCmdContDO.setCreationDate(now);
        pmCmdContDO.setLastUpdateDate(now);
        pmCmdContDOList.add(pmCmdContDO);
    }

    //命令小板表
    public void SetPmCmdSmallPlt(PmSmallPltPosDO pmSmallPltPosDO) {
        PmCmdSmallPltDO pmCmdSmallPltDO = new PmCmdSmallPltDO();
        pmCmdSmallPltDO.setPonoSlabNo(pmSmallPltPosDO.getPdMatNo());
        pmCmdSmallPltDO.setTmpSlabNo(pmSmallPltPosDO.getPdMatNo());
        pmCmdSmallPltDO.setSmallPlateDivNo(pmSmallPltPosDO.getSmallPlateDivNo());
        pmCmdSmallPltDO.setOrderNo(pmSmallPltPosDO.getOrderNo());
        pmCmdSmallPltDO.setOrderSeqNo(Long.valueOf(pmSmallPltPosDO.getOrderSeqNo()));
        pmCmdSmallPltDO.setSlabNo(pmSmallPltPosDO.getSlabNo());
        pmCmdSmallPltDO.setPickModeWidDir(pmSmallPltPosDO.getPickModeWidDir());
        pmCmdSmallPltDO.setPickModeLenDir(pmSmallPltPosDO.getPickModeLenDir());
        pmCmdSmallPltDO.setOrderThick(pmSmallPltPosDO.getOrderThick());
        pmCmdSmallPltDO.setOrderWidth(pmSmallPltPosDO.getOrderWidth());
        pmCmdSmallPltDO.setOrderLen(BigDecimal.valueOf(pmSmallPltPosDO.getOrderLen()));
        pmCmdSmallPltDO.setOrderLenMin(BigDecimal.valueOf(pmSmallPltPosDO.getOrderLenMin()));
        pmCmdSmallPltDO.setOrderLenMax(BigDecimal.valueOf(pmSmallPltPosDO.getOrderLenMax()));
        pmCmdSmallPltDO.setSmallPlateThick(pmSmallPltPosDO.getSmallPlateThick());
        pmCmdSmallPltDO.setSmallPlateWidth(pmSmallPltPosDO.getSmallPlateWidth());
        pmCmdSmallPltDO.setSmallPlateLen(BigDecimal.valueOf(pmSmallPltPosDO.getSmallPlateLen()));
        pmCmdSmallPltDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
        pmCmdSmallPltDO.setLastUpdatedBy(781L);
        pmCmdSmallPltDO.setCreationDate(now);
        pmCmdSmallPltDO.setLastUpdateDate(now);
        pmCmdSmallPltDOList.add(pmCmdSmallPltDO);
    }

}
